/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */
package org.greenstone.gatherer.cdm;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import org.greenstone.gatherer.Gatherer;
import org.greenstone.gatherer.cdm.CollectionMeta;
import org.greenstone.gatherer.cdm.CommandTokenizer;
import org.greenstone.gatherer.cdm.Control;
import org.greenstone.gatherer.cdm.DOMProxyListModel;
import org.greenstone.gatherer.cdm.Index;
import org.greenstone.gatherer.gui.GComboBox;
import org.greenstone.gatherer.msm.ElementWrapper;
import org.greenstone.gatherer.msm.MSMUtils;
import org.greenstone.gatherer.util.ExclusiveListSelectionListener;
import org.greenstone.gatherer.util.Utility;
import org.w3c.dom.*;
/** This class is resposible for storing the indexes which have been assigned to this collection and the default index, and providing methods for interacting with both these data pools. It also knows how to turn itself into a String as it would be displayed in the collection configuration file.
 * @author John Thompson, Greenstone Digital Library, University of Waikato
 * @version 2.3
 */
public class IndexManager
    extends DOMProxyListModel {

    static final private Dimension LABEL_SIZE = new Dimension(150,30);
    static final private String MGINDEXES = "mg indexes";
    static final private String MGPPINDEXES = "mgpp indexes";

    /** The controls for editing the indexes. */
    private Control controls = null;
    /** A model of the levels, also based on the DOM. */
    private DOMProxyListModel levels_model = null;
    /** A reference to ourselves so our inner methods have access. */
    private DOMProxyListModel model = null;
    /** The default index. */
    private Index default_index = null;

    /** Constructor. */
    public IndexManager(Element indexes) {
	super(indexes, CollectionConfiguration.INDEX_ELEMENT, new Index());
	Gatherer.println("IndexManager: " + getSize() + " indexes parsed.");
	model = this;
	// Parse and retrieve the default index
	NodeList default_index_elements = CollectionDesignManager.collect_config.getDocumentElement().getElementsByTagName(CollectionConfiguration.INDEX_DEFAULT_ELEMENT);
	if(default_index_elements.getLength() > 0) {
	    default_index = new Index((Element)default_index_elements.item(0));
	}
	// Parse and retrieve the levels element
	Element levels_element = CollectionDesignManager.collect_config.getLevels();
	levels_model = new DOMProxyListModel(levels_element, CollectionConfiguration.CONTENT_ELEMENT, new Level());
	Gatherer.println(" + " + levels_model.getSize() + " levels parsed.");
    }
    /** Method to add a new index.
     * @param index The <strong>Index</strong> to add.
     * @see org.greenstone.gatherer.Gatherer
     * @see org.greenstone.gatherer.collection.CollectionManager
     */
    public void addIndex(Index index, CollectionMeta metadatum) {
	///ystem.err.println("Adding an index: " + index.toString());
	if(!contains(index)) {
	    CollectionDesignManager.collectionmeta_manager.addMetadatum(metadatum);
	    // Retrieve the currently last index
	    if(getSize() > 0) {
		Index last_index = (Index)getElementAt(getSize() - 1);
		addAfter(index, last_index);
	    }
	    else {
		add(index);
	    }
	    Gatherer.c_man.configurationChanged();
	}
	else {
	    JOptionPane.showMessageDialog(Gatherer.g_man, get("CDM.IndexManager.Index_Exists"), get("General.Warning"), JOptionPane.WARNING_MESSAGE);
	}
    }

    public void addLevel(Level level, CollectionMeta metadatum) {
	if(!levels_model.contains(level)) {
	    CollectionDesignManager.collectionmeta_manager.addMetadatum(metadatum);
	    // Retrieve the currently last index
	    if(getSize() > 0) {
		Level last_level = (Level)levels_model.getElementAt(levels_model.getSize() - 1);
		levels_model.addAfter(level, last_level);
	    }
	    else {
		add(level);
	    }
	    Gatherer.c_man.configurationChanged();
	}
	else {
	    JOptionPane.showMessageDialog(Gatherer.g_man, get("CDM.IndexManager.Level_Exists"), get("General.Warning"), JOptionPane.WARNING_MESSAGE);
	}
    }

    public void destroy() {
	if(controls != null) {
	    controls.destroy();
	    controls = null;
	}
	default_index = null;
	model = null;
    }

    /** Method to acquire the controls for editing the indexes.
     * @return the Control
     */
    public Control getControls() {
	if(controls == null) {
	    // Build controls
	    controls = new IndexControl();
	}
	return controls;
    }

    /** Method to get the default index.
     * @return The default <strong>Index</strong>.
     */
    public Index getDefault() {
	if(default_index != null && default_index.isAssigned()) {
	    return default_index;
	}
	else {
	    return null;
	}
    }

    /** Method to retrieve a certain index, as referenced by an index number.
     * @param index An <i>int</i> which indicates the position of the desired index.
     * @return The <strong>Index</strong> at the given index, or <i>null</i> if no such index exists.
     */
    public Index getIndex(int index) {
	if(0 <= index && index < getSize()) {
	    return (Index)getElementAt(index);
	}
	return null;
    }

    /** Method to retrieve a certain index, given its id.
     * @param id the id of the index as a String
     * @return the Index that matches id, or null if no such index exists
     */
    public Index getIndex(String id) {
	int size = getSize();
	for(int i = 0; i < size; i++) {
	    Index index = (Index) getElementAt(i);
	    if(index.getID().equals(id)) {
		return index;
	    }
	}
	return null;
    }

    public ArrayList getIndexes() {
	return children();
    }

    public Level getLevel(String name) {
	int levels_model_size = levels_model.getSize();
	for(int i = 0; i < levels_model_size; i++) {
	    Level level = (Level) levels_model.getElementAt(i);
	    if(level.getName().equals(name)) {
		return level;
	    }
	}
	return null;
    }

    public void moveIndex(Index index, boolean move_up) {
	// Determine the indexes current position
	int position = indexOf(index);
	// Determine if it can be moved, ie if its not already at the top trying to move up, or at the bottom trying to move down.
	if(position == -1) {
	    return;
	}
	if(position == 0 && move_up) {
	    return;
	}
	if(position == (getSize()) - 1 && !move_up) {
	    return;
	}
	// Ok, move the index
	if(move_up) {
	    // Retrieve the index at position - 1
	    Index previous_index = (Index) getElementAt(position - 1);
	    // And add before. This will automatically remove the index first, as an Element can only exist once in a particular document
	    addBefore(index, previous_index);
	}
	else {
	    // Retrieve the index at position + 1
	    Index next_index = (Index) getElementAt(position + 1);
	    // And add after. This will automatically remove the index first, as an Element can only exist once in a particular document
	    addAfter(index, next_index);
	}
	// Schedule the collection for saving
	Gatherer.c_man.configurationChanged();
    }

    public void moveLevel(Level level, boolean move_up) {
	// Determine the leveles current position
	int position = levels_model.indexOf(level);
	// Determine if it can be moved, ie if its not already at the top trying to move up, or at the bottom trying to move down.
	if(position == -1) {
	    return;
	}
	if(position == 0 && move_up) {
	    return;
	}
	if(position == (levels_model.getSize()) - 1 && !move_up) {
	    return;
	}
	// Ok, move the level
	if(move_up) {
	    // Retrieve the level at position - 1
	    Level previous_level = (Level) levels_model.getElementAt(position - 1);
	    // And add before. This will automatically remove the level first, as an Element can only exist once in a particular document
	    levels_model.addBefore(level, previous_level);
	}
	else {
	    // Retrieve the level at position + 1
	    Level next_level = (Level) levels_model.getElementAt(position + 1);
	    // And add after. This will automatically remove the level first, as an Element can only exist once in a particular document
	    levels_model.addAfter(level, next_level);
	}
	// Schedule the collection for saving
	Gatherer.c_man.configurationChanged();
    }

    /** Method to remove a certain index.
     * @param index the Index to remove.
     * @see org.greenstone.gatherer.Gatherer
     * @see org.greenstone.gatherer.cdm.CollectionDesignManager
     * @see org.greenstone.gatherer.cdm.CollectionMetaManager
     * @see org.greenstone.gatherer.collection.CollectionManager
     */
    public void removeIndex(Index index) {
	if(index != null) {
	    // Remove any current metadata from this index
	    CollectionDesignManager.collectionmeta_manager.removeMetadata(CollectionConfiguration.STOP_CHARACTER + index.getID());
	    // Check if the index removed happens to be the default index
	    if(default_index != null && default_index.equals(index)) {
		default_index.setAssigned(false);
	    }
	    // Remove the index
	    remove(index);
	    Gatherer.c_man.configurationChanged();
	}
    }

    public void removeLevel(Level level) {
	if(level != null) {
	    // Remove any current metadata from this level
	    CollectionDesignManager.collectionmeta_manager.removeMetadata(CollectionConfiguration.STOP_CHARACTER + level.getName());
	    // Remove the level
	    levels_model.remove(level);
	    Gatherer.c_man.configurationChanged();
	}
    }

    /** Method to set the default index.
     * @param index the new default Index
     * @see org.greenstone.gatherer.Gatherer
     * @see org.greenstone.gatherer.collection.CollectionManager
     */
    public void setDefault(Index index) {
	if(index != null) {
	    if(default_index == null) {
		// Create the default index element, and place immediately after indexes element.
		Element default_index_element = root.getOwnerDocument().createElement(CollectionConfiguration.INDEX_DEFAULT_ELEMENT);
		default_index = new Index(default_index_element);
		Node target_node = CollectionConfiguration.findInsertionPoint(default_index_element);
		if(target_node != null) {
		    root.getOwnerDocument().getDocumentElement().insertBefore(default_index_element, target_node);
		}
		else {
		    root.getOwnerDocument().getDocumentElement().appendChild(default_index_element);
		}
	    }
	    default_index.setAssigned(true);
	    default_index.setLevel(index.getLevel());
	    default_index.setSources(index.getSources());
	}
	else {
	    if(default_index != null) {
		default_index.setAssigned(false);
	    }
	}
	Gatherer.c_man.configurationChanged();
    }
    
    /** This method is reponsible for changing the underlying Index commands from MG to MGPP and back again. This turns out to be easyish for MG->MGPP and very hard for the reverse. For the former we remove the level fragment and make sure the same levels are set, then we produce a list of the sources involved, breaking down comma seperated lists and making sure each item it unique. Changing back the other way turns out to be impossible, so we don't (beyond generating document:text, section:text and paragraph:text if text is an index and the respective levels are present). In either case we start by creating a comment containing the old index information.
     * @param  state true to enable MGPP indexes, false to use standard MG style ones
     */
    public void setMGPPEnabled(boolean state) {
	if(state != root.getAttribute(CollectionConfiguration.MGPP_ATTRIBUTE).equals(CollectionConfiguration.TRUE_STR)) {
	    if(state) {
		Element mg_element = root;
		// Retrieve and assign the MGPP indexes element.
		Element mgpp_element = CollectionDesignManager.collect_config.getMGPPIndexes();
		mgpp_element.setAttribute(CollectionConfiguration.ASSIGNED_ATTRIBUTE, CollectionConfiguration.TRUE_STR);
		levels_model.root.setAttribute(CollectionConfiguration.ASSIGNED_ATTRIBUTE, CollectionConfiguration.TRUE_STR);
		// If the MGPP indexes element is empty (ie was created by CollectionConfiguration), generate new MGPP index from the existing index
		NodeList indexes = mgpp_element.getElementsByTagName(CollectionConfiguration.INDEX_ELEMENT);
		if(indexes.getLength() == 0) {
		    ArrayList levels_list = new ArrayList();
		    ArrayList sources_list = new ArrayList();
		    // We first use details from the default index if any
		    if(default_index != null) {
			int level_int = default_index.getLevel();
			if(0 <= level_int && level_int < 3) {
			    String level = Index.LEVEL[level_int];
			    if(!levels_list.contains(level)) {
				levels_list.add(level);
			    }
			    level = null;
			}
			ArrayList sources = default_index.getSources();
			sources.removeAll(sources_list);
			sources_list.addAll(sources);
		    }
		    int size = getSize();
		    for(int i = 0; i < size; i++) {
			Index index = (Index) getElementAt(i);
			int level_int = index.getLevel();
			if(0 <= level_int && level_int < 3) {
			    String level = Index.LEVEL[level_int];
			    if(!levels_list.contains(level)) {
				levels_list.add(level);
			    }
			    level = null;
			}
			ArrayList sources = index.getSources();
			sources.removeAll(sources_list);
			sources_list.addAll(sources);
			index = null;
		    }
		    // Replace mg element with mgpp element 
		    setRoot(mgpp_element);

		    // We now have a list of sources and a list of levels, so create new indexes and levels based on these
		    int sources_list_size = sources_list.size();
		    for(int j = 0; j < sources_list_size; j++) {
			Object source_object = sources_list.get(j);
			String source_str = null;
			if(source_object instanceof ElementWrapper) {
			    source_str = ((ElementWrapper)source_object).getName();
			}
			else {
			    source_str = source_object.toString();
			}
			ArrayList new_sources = new ArrayList();
			new_sources.add(source_object);
			source_object = null;
			Index new_index = new Index(new_sources);
			// Try to retrieve existing metadatum
			source_str = new_index.getID();
			CollectionMeta metadatum = CollectionDesignManager.collectionmeta_manager.getMetadatum(CollectionConfiguration.STOP_CHARACTER + source_str, false);
			// If no metadata was found, add new pseudo metadata using the id
			if(metadatum == null) {
			    metadatum = new CollectionMeta(CollectionConfiguration.STOP_CHARACTER + source_str);
			    metadatum.setAssigned(true);
			    metadatum.setValue(source_str);
			}
			// If it was found, ensure it is assigned
			else {
			    metadatum.setAssigned(true);
			}
			source_str = null;
			addIndex(new_index, metadatum);
			metadatum = null;
			new_index = null;
			new_sources = null;
			source_str = null;
		    }
		    int levels_list_size = levels_list.size();
		    for(int k = 0; k < levels_list_size; k++) {
			Level new_level = new Level((String)levels_list.get(k));
			if(!levels_model.contains(new_level)) {
			    levels_model.add(levels_model.getSize(), new_level);
			}
			new_level = null;
		    }
		}
		else {
		    // Replace mg element with mgpp element 
		    setRoot(mgpp_element);
		}
		// Unassign MG element
		mg_element.setAttribute(CollectionConfiguration.ASSIGNED_ATTRIBUTE, CollectionConfiguration.FALSE_STR);
		mg_element = null;
	    }
	    else {
		Element mgpp_element = root;
		// Retrieve and assign MG element and default index element
		Element mg_element = CollectionDesignManager.collect_config.getMGIndexes();
		mg_element.setAttribute(CollectionConfiguration.ASSIGNED_ATTRIBUTE, CollectionConfiguration.TRUE_STR);
		if(default_index != null) {
		    default_index.getElement().setAttribute(CollectionConfiguration.ASSIGNED_ATTRIBUTE, CollectionConfiguration.TRUE_STR);
		}
		// If mg element has no indexes, and the current mgpp index include a text one, then generate text indexes for each of the registered levels. 
		NodeList indexes = mgpp_element.getElementsByTagName(CollectionConfiguration.INDEX_ELEMENT);
		if(indexes.getLength() == 0) {
		    Index index = getIndex(CollectionConfiguration.TEXT_STR);
		    if(index != null) {
			// Replace mgpp element with mg element
			setRoot(mg_element);
			int level_size = levels_model.getSize();
			for(int i = 0; i < level_size; i++) {
			    Level level = (Level) levels_model.getElementAt(i);
			    Index new_index = new Index(level.getName(), index.getSources());
			    // Try to retrieve existing metadatum
			    String source_str = new_index.getID();
			    CollectionMeta metadatum = CollectionDesignManager.collectionmeta_manager.getMetadatum(CollectionConfiguration.STOP_CHARACTER + source_str, false);
			    // If no metadata was found, add new pseudo metadata using the id
			    if(metadatum == null) {
				metadatum = new CollectionMeta(CollectionConfiguration.STOP_CHARACTER + source_str);
				metadatum.setAssigned(true);
				metadatum.setValue(source_str);
			    }
			    // If it was found, ensure it is assigned
			    else {
				metadatum.setAssigned(true);
			    }
			    source_str = null;
			    addIndex(new_index, metadatum);
			    new_index = null;
			    level = null;
			}
		    }
		}
		else {
		    // Replace mgpp element with mg element
		    setRoot(mg_element);
		}
		// Unassign mgpp element and levels
		mgpp_element.setAttribute(CollectionConfiguration.ASSIGNED_ATTRIBUTE, CollectionConfiguration.FALSE_STR);
		levels_model.root.setAttribute(CollectionConfiguration.ASSIGNED_ATTRIBUTE, CollectionConfiguration.FALSE_STR);
	    }
	}
    }

    /** Overloaded to call get with both a key and an empty argument array.
     * @param key A <strong>String</strong> which is mapped to a initial String within the ResourceBundle.
     * @return A <strong>String</strong> which has been referenced by the key String and that either contains no argument fields, or has had the argument fields automatiically populated with formatting Strings of with argument String provided in the get call.
     */
    private String get(String key) {
	return get(key, null);
    }

    /** Used to retrieve a property value from the Locale specific ResourceBundle, based upon the key and arguments supplied. If the key cannot be found or if some other part of the call fails a default (English) error message is returned. <BR>
     * Here the get recieves a second argument which is an array of Strings used to populate argument fields, denoted {<I>n</I>}, within the value String returned. Note that argument numbers greater than or equal to 32 are automatically mapped to the formatting String named Farg<I>n</I>.
     * @param key A <strong>String</strong> which is mapped to a initial String within the ResourceBundle.
     * @param args A <strong>String[]</strong> used to populate argument fields within the complete String.
     * @return A <strong>String</strong> which has been referenced by the key String and that either contains no argument fields, or has had the argument fields automatically populated with formatting Strings of with argument String provided in the get call.
     * @see org.greenstone.gatherer.Gatherer
     * @see org.greenstone.gatherer.Dictionary
     */
    private String get(String key, String args[]) {
	if(key.indexOf('.') == -1) {
	    key = "CDM.IndexManager." + key;
	}
	return Gatherer.dictionary.get(key, args);
    }

    /** This class creates a set of controls for editing the indexes. */
    private class IndexControl
	extends JPanel 
	implements Control {

	private CardLayout card_layout;

	/** The default size of a label on this control. */
	private Dimension LABEL_SIZE = new Dimension(125,25);
	/** The button used to add a new index. */
	private JButton add = null;
	/** The button used to clear the default index. */
	private JButton clear_default = null;
	/** The button used to remove an index. */
	private JButton remove = null;
	/** The button used to set the default index. */
	private JButton set_default = null;
	/** The combobox used to adjust what level the new index will built on. */
	private JComboBox level = null;
	/** The label denoting the default index to be built for this collection. */
	private JLabel default_label = null;
	/** The label denoting the list of assigned indexes. */
	private JLabel index_label = null;
	/** The label denoting the control for choosing index level. */
	private JLabel level_label = null;
	/** The label denoting the name of the index. */
	private JLabel name_label = null;
	/** The label denoting the list of possible sources for the index. */
	private JLabel source_label = null;
	/** The title shown at the top of the index controls. */
	private JLabel title = null;
	/** The list of assigned indexes. */
	private JList index_list = null;
	/** The list of possible sources on which the index could be built. */
	private JList source_list = null;
	/** The panel used to display the list of assigned indexes. */
	private JPanel assigned_pane = null;
	/** The panel which contains the buttons used to edit indexes. */
	private JPanel button_pane = null;
	/** The panel on which all other panels are displayed. */
	private JPanel central_pane = null;
	/** The control pane showing the editable controls. */
	private JPanel control_pane = null;
	/** The edit pane contains the list of possible sources. */
	private JPanel edit_pane = null;
	/** A panel used to correctly lay out the default index label. */
	private JPanel default_pane = null;
	/** The panel containing the title label and instructions. */
	private JPanel header_pane = null;
	/** The lvel panel contains the control for adjusting index level. */
	private JPanel level_pane = null;
	/** The pane in which the name control is set. */
	private JPanel name_pane = null;
	/** The scrollpane containing the instructions text area. */
	private JScrollPane instructions_scroll = null;
	/** A text area containing inline instructions. */
	private JTextArea instructions = null;
	/** A text field naming the default index. Non-editable. */
	private JTextField default_value = null;
	/** A text field for controlling the name of the new index. */
	private JTextField name = null;

	private MGPPControl mgppindexes_control = null;

	/** Constructor.
	 * @see org.greenstone.gatherer.Configuration
	 * @see org.greenstone.gatherer.Gatherer
	 * @see org.greenstone.gatherer.cdm.IndexManager.Control.AddListener
	 * @see org.greenstone.gatherer.cdm.IndexManager.Control.ClearDefaultListener
	 * @see org.greenstone.gatherer.cdm.IndexManager.Control.NameListener
	 * @see org.greenstone.gatherer.cdm.IndexManager.Control.RemoveListener
	 * @see org.greenstone.gatherer.cdm.IndexManager.Control.SetDefaultListener
	 * @see org.greenstone.gatherer.collection.CollectionManager
	 * @see org.greenstone.gatherer.gui.Coloring
	 * @see org.greenstone.gatherer.msm.MetadataSetManager
	 * @see org.greenstone.gatherer.util.ExclusiveListSelectionListener
	 */
	public IndexControl() {
	    super();
	    // Creation
	    JPanel mgindexes_panel = new JPanel();
	    card_layout = new CardLayout();

	    add = new JButton(get("Add"));
	    add.setEnabled(false);
	    add.setMnemonic(KeyEvent.VK_A);
	    assigned_pane = new JPanel();
	    button_pane = new JPanel();
	    central_pane = new JPanel();
	    control_pane = new JPanel();
	    clear_default = new JButton(get("Clear_Default"));
	    clear_default.setMnemonic(KeyEvent.VK_C);
	    if(default_index == null) {
		clear_default.setEnabled(false);
	    }
	    default_label = new JLabel(get("Default_Index"));
	    default_pane = new JPanel();
	    if(default_index != null) {
		default_value = new JTextField(default_index.toString());
		default_value.setCaretPosition(0);
	    }
	    else {
		default_value = new JTextField();
	    }
	    default_value.setBackground(Gatherer.config.getColor("coloring.collection_tree_background", false));
	    default_value.setEditable(false);
	    edit_pane = new JPanel();
	    header_pane = new JPanel();
	    index_label = new JLabel(get("Indexes"));
	    index_list = new JList(model);
	    instructions = new JTextArea(get("Instructions"));
	    instructions.setEditable(false);
	    instructions.setLineWrap(true);
	    instructions.setRows(5);
	    instructions.setWrapStyleWord(true);
	    level = new JComboBox();
	    level.addItem(get("Document"));
	    level.addItem(get("Paragraph"));
	    level.addItem(get("Section"));
	    level.setEditable(false);
	    level_label = new JLabel(get("Level"));
	    level_label.setPreferredSize(LABEL_SIZE);
	    level_pane = new JPanel();
	    name = new JTextField();
	    name_label = new JLabel(get("Name"));
	    name_label.setPreferredSize(LABEL_SIZE);
	    name_pane = new JPanel();
	    remove = new JButton(get("Remove"));
	    remove.setEnabled(false);
	    remove.setMnemonic(KeyEvent.VK_R);
	    set_default = new JButton(get("Set_Default"));
	    set_default.setEnabled(false);
	    set_default.setMnemonic(KeyEvent.VK_S);
	    source_label = new JLabel(get("Source"));
	    source_list = new JList();
	    title = new JLabel(get("Title"));
	    title.setHorizontalAlignment(JLabel.CENTER);

	    // This class is getting a little crowded, so I'll generate the many mgpp controls in a inner class.
	    mgppindexes_control = new MGPPControl();

	    // Listeners
	    add.addActionListener(new AddListener());
	    clear_default.addActionListener(new ClearDefaultListener());
	    remove.addActionListener(new RemoveListener());
	    set_default.addActionListener(new SetDefaultListener());
	    name.addKeyListener(new NameListener());
	    ListListener ll = new ListListener();
	    index_list.addListSelectionListener(ll);
	    source_list.addListSelectionListener(ll);
	    ExclusiveListSelectionListener elsl = new ExclusiveListSelectionListener();
	    elsl.add(index_list);
	    elsl.add(source_list);

	    // Layout
	    title.setBorder(BorderFactory.createEmptyBorder(0,0,2,0));

	    instructions.setBorder(BorderFactory.createEmptyBorder(2,5,2,5));

	    header_pane.setLayout(new BorderLayout());
	    header_pane.add(title, BorderLayout.NORTH);
	    header_pane.add(new JScrollPane(instructions), BorderLayout.CENTER);

	    name_label.setBorder(BorderFactory.createEmptyBorder(0,0,0,5));

	    name_pane.setBorder(BorderFactory.createEmptyBorder(4,2,4,2));
	    name_pane.setLayout(new BorderLayout());
	    name_pane.add(name_label, BorderLayout.WEST);
	    name_pane.add(name, BorderLayout.CENTER);

	    control_pane.setLayout(new BorderLayout());
	    control_pane.add(source_label, BorderLayout.NORTH);
	    control_pane.add(new JScrollPane(source_list), BorderLayout.CENTER);

	    level_label.setBorder(BorderFactory.createEmptyBorder(0,0,0,5));

	    level_pane.setBorder(BorderFactory.createEmptyBorder(4,2,4,2));
	    level_pane.setLayout(new BorderLayout());
	    level_pane.add(level_label, BorderLayout.WEST);
	    level_pane.add(level, BorderLayout.CENTER);

	    edit_pane.setBorder(BorderFactory.createEmptyBorder(0,0,0,5));
	    edit_pane.setLayout(new BorderLayout());
	    edit_pane.add(name_pane, BorderLayout.NORTH);
	    edit_pane.add(control_pane, BorderLayout.CENTER);
	    edit_pane.add(level_pane, BorderLayout.SOUTH);

	    default_label.setBorder(BorderFactory.createEmptyBorder(0,0,0,5));

	    default_pane.setBorder
		(BorderFactory.createCompoundBorder
		 (BorderFactory.createCompoundBorder
		  (BorderFactory.createEmptyBorder(2,2,2,2),
		   BorderFactory.createRaisedBevelBorder()),
		  BorderFactory.createEmptyBorder(2,2,2,2)));
	    default_pane.setLayout(new BorderLayout());
	    default_pane.add(default_label, BorderLayout.WEST);
	    default_pane.add(default_value, BorderLayout.CENTER);

	    assigned_pane.setLayout(new BorderLayout());
	    assigned_pane.add(index_label, BorderLayout.NORTH);
	    assigned_pane.add(new JScrollPane(index_list), BorderLayout.CENTER);
	    assigned_pane.add(default_pane, BorderLayout.SOUTH);

	    central_pane.setBorder(BorderFactory.createEmptyBorder(2,0,2,0));
	    central_pane.setLayout(new GridLayout(1,2));
	    central_pane.add(edit_pane);
	    central_pane.add(assigned_pane);

	    button_pane.setLayout(new GridLayout(2,2));
	    button_pane.add(add);
	    button_pane.add(remove);
	    button_pane.add(clear_default);
	    button_pane.add(set_default);

	    mgindexes_panel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
	    mgindexes_panel.setLayout(new BorderLayout());
	    mgindexes_panel.add(header_pane, BorderLayout.NORTH);
	    mgindexes_panel.add(central_pane, BorderLayout.CENTER);
	    mgindexes_panel.add(button_pane, BorderLayout.SOUTH);

	    setLayout(card_layout);
	    add(mgindexes_panel, MGINDEXES);
	    add(mgppindexes_control, MGPPINDEXES);
	}
	/* Destructor, removes persistant listeners from the Dictionary.
	 */
	public void destroy() {
	    mgppindexes_control.destroy();
	    mgppindexes_control = null;
	}

	public void gainFocus() {
	    boolean mgpp_enabled = CollectionDesignManager.searchtype_manager.isMGPPEnabled();
	    if(instructions != null) {
		instructions.setCaretPosition(0);
	    }
	    if(mgpp_enabled) {
		mgppindexes_control.gainFocus();
	    }
	    else {
		// Reload the assigned indexes list.
		ArrayList new_data = new ArrayList();
		new_data.add(CollectionConfiguration.TEXT_STR);
		new_data.addAll(Gatherer.c_man.getCollection().msm.getAssignedElements());
		// reset the model in the list and combobox
		source_list.setListData(new_data.toArray());
		new_data = null;
	    }
	    // Bring the appropriate card to the front
	    if(mgpp_enabled) {
		card_layout.show(this, MGPPINDEXES);
	    }
	    else {
		card_layout.show(this, MGINDEXES);
	    }
	}

	public void loseFocus() {
	}

	/** Listens for actions apon the 'add' button in the IndexManager controls, and if detected calls the add method of the manager with a newly created index. */
	private class AddListener
	    implements ActionListener {
	    /** Method called when an action is performed on a registered component, and when it does we check if we have enough data to create a new index, and if so we create one.
	     * @param event An <strong>ActionEvent</strong> providing extra information about the event.
	     * @see org.greenstone.gatherer.cdm.CollectionDesignManager
	     * @see org.greenstone.gatherer.cdm.CollectionMeta
	     * @see org.greenstone.gatherer.cdm.CollectionMetaManager
	     * @see org.greenstone.gatherer.cdm.Index
	     * @see org.greenstone.gatherer.cdm.Language
	     * @see org.greenstone.gatherer.cdm.LanguageManager
	     */
	    public void actionPerformed(ActionEvent event) {
		if(!source_list.isSelectionEmpty() && name.getText().length() != 0) {
		    Object object[] = source_list.getSelectedValues();
		    ArrayList sources = new ArrayList();
		    for(int i = 0; i < object.length; i++) {
			sources.add(object[i]);
		    }
		    object = null;
		    Index index = new Index(level.getSelectedIndex(), sources);
		    sources = null;
		    // Before we add the index to the model, we have to add the collection metadata for this.
		    CollectionMeta metadatum = new CollectionMeta(CollectionConfiguration.STOP_CHARACTER + index.getID());
		    metadatum.setValue(name.getText());
		    // Finally add index.
		    addIndex(index, metadatum);
		    metadatum = null;
		    index = null;
		}
	    }
	}
	/** Listens for actions apon the 'clear default' button in the IndexManager controls, and if detected calls the setDefault method of the manager with <i>null</i>. */
	private class ClearDefaultListener
	    implements ActionListener {
	    /** If called when an action occurs on a registered component, we clear the default index.
	     * @param event An <strong>ActionEvent</strong> containing extra information about the action that occured.
	     */
	    public void actionPerformed(ActionEvent event) {
		setDefault(null);
		clear_default.setEnabled(false);
		default_value.setText("");
	    }
	}
	/** Listens for actions apon the 'remove' button in the IndexManager controls, and if detected calls the remove method of the manager with the index selected for removal. */
	private class RemoveListener
	    implements ActionListener {
				/** If called when an action occurs on a registered component, we remove the currently selected index, if there is one.
				 * @param event An <strong>ActionEvent</strong> containing extra information about the action that occured.
				 * @see org.greenstone.gatherer.cdm.Index
				 */
	    public void actionPerformed(ActionEvent event) {
		if(!index_list.isSelectionEmpty()) {
		    removeIndex((Index)index_list.getSelectedValue());
		    if(default_index == null) {
			clear_default.setEnabled(false);
			default_value.setText("");
		    }
		}
	    }
	}

	/** Listens for selections within the list on the IndexManager controls, and if a change is detected enables, or disables, controls appropriately. */
	private class ListListener
	    implements ListSelectionListener {
				/** This method is called whenever the source list selection changes. When it does we need to check if the add button should now be enabled.
				 * @param event A <strong>ListSelectionEvent</strong> containing further information about the list selection.
				*/
	    public void valueChanged(ListSelectionEvent event) {
		if(source_list.isSelectionEmpty() || name.getText().length() == 0) {
		    add.setEnabled(false);
		}
		else {
		    add.setEnabled(true);
		}
		if(index_list.isSelectionEmpty()) {
		    remove.setEnabled(false);
		    set_default.setEnabled(false);
		}
		else {
		    remove.setEnabled(true);
		    set_default.setEnabled(true);
		}
	    }
	}

	/** Listens for key presses within the name field, and enabled or disables controls as appropriate. */
	private class NameListener
	    extends KeyAdapter {
				/** Called when a key is released, this is the perfect time to enable the add button if the fields are appropriately set.
				 * @param event A <strong>KeyEvent</strong> containing information about the key released.
				*/
	    public void keyReleased(KeyEvent event) {
		if(source_list.isSelectionEmpty() || name.getText().length() == 0) {
		    add.setEnabled(false);
		}
		else {
		    add.setEnabled(true);
		}
		if(index_list.isSelectionEmpty()) {
		    remove.setEnabled(false);
		    set_default.setEnabled(false);
		}
		else {
		    remove.setEnabled(true);
		    set_default.setEnabled(true);
		}
	    }
	}

	/** Listens for actions apon the 'set default' button in the IndexManager controls, and if detected calls the setDefault method of the manager with the index selected for default. */
	private class SetDefaultListener
	    implements ActionListener {
				/** If called when an action occurs on a registered component, we set the default index to the index currently selected.
				 * @param event An <strong>ActionEvent</strong> containing extra information about the action that occured.
				 * @see org.greenstone.gatherer.cdm.Index
				 */
	    public void actionPerformed(ActionEvent event) {
		setDefault((Index)index_list.getSelectedValue());
		if(default_index != null) {
		    clear_default.setEnabled(true);
		    default_value.setText(default_index.toString());
		    default_value.setCaretPosition(0);
		}
		else {
		    clear_default.setEnabled(false);
		    default_value.setText("");
		}
	    }
	}
    }

    private class MGPPControl
	extends JPanel {

	private GComboBox index_combobox;
	private GComboBox level_combobox;
	private JButton add_index_button;
	private JButton add_level_button;
	private JButton move_index_down_button;
	private JButton move_level_down_button;
	private JButton move_index_up_button;
	private JButton move_level_up_button;
	private JButton remove_index_button;
	private JButton remove_level_button;
	private JLabel current_levels_label;
	private JLabel current_indexes_label;
	private JLabel index_label;
	private JLabel index_name_label;
	private JLabel level_label;
	private JLabel level_name_label;
	private JLabel move_index_down_label;
	private JLabel move_level_down_label;
	private JLabel move_index_up_label;
	private JLabel move_level_up_label;
	private JLabel title_label;
	private JList current_levels_list;
	private JList current_indexes_list;
	private JTabbedPane tabbed_pane;
	private JTextArea instructions_textarea;
	private JTextField index_name_field;
	private JTextField level_name_field;

	public MGPPControl() {
	    // Create Indexes
	    JPanel indexes_panel = new JPanel();
	    
	    JPanel current_indexes_panel = new JPanel();

	    current_indexes_label = new JLabel("CDM.IndexManager.MGPP.Current_Indexes");

	    current_indexes_list = new JList(model);
	    current_indexes_list.setVisibleRowCount(5);

	    JPanel index_movement_panel = new JPanel();

	    ImageIcon move_index_up_icon = Utility.getImage("arrow-up.gif");
	    move_index_up_button = new JButton();
	    move_index_up_label = new JLabel("CDM.Move.Move_Up");
	    move_index_up_label.setHorizontalAlignment(JLabel.CENTER);
	    move_index_up_label.setPreferredSize(LABEL_SIZE);
	    move_index_up_button.setLayout(new BorderLayout());
	    move_index_up_button.add(new JLabel(move_index_up_icon), BorderLayout.WEST);
	    move_index_up_button.add(move_index_up_label, BorderLayout.CENTER);
	    move_index_up_button.add(new JLabel(move_index_up_icon), BorderLayout.EAST);
	    move_index_up_button.setMnemonic(KeyEvent.VK_U);

	    ImageIcon move_index_down_icon = Utility.getImage("arrow-down.gif");
	    move_index_down_button = new JButton();
	    move_index_down_label = new JLabel("CDM.Move.Move_Down");
	    move_index_down_label.setHorizontalAlignment(JLabel.CENTER);
	    move_index_down_label.setPreferredSize(LABEL_SIZE);
	    move_index_down_button.setLayout(new BorderLayout());
	    move_index_down_button.add(new JLabel(move_index_down_icon), BorderLayout.WEST);
	    move_index_down_button.add(move_index_down_label, BorderLayout.CENTER);
	    move_index_down_button.add(new JLabel(move_index_down_icon), BorderLayout.EAST);
	    move_index_down_button.setMnemonic(KeyEvent.VK_D);

	    JPanel index_spacer_panel = new JPanel();

	    JPanel index_body_panel = new JPanel();

	    JPanel index_details_panel = new JPanel();

	    JPanel index_name_panel = new JPanel();

	    index_name_label = new JLabel("CDM.IndexManager.MGPP.Index_Name");
	    index_name_label.setPreferredSize(LABEL_SIZE);

	    index_name_field = new JTextField();

	    JPanel index_panel = new JPanel();

	    index_label = new JLabel("CDM.IndexManager.MGPP.Index");
	    index_label.setPreferredSize(LABEL_SIZE);

	    index_combobox = new GComboBox();
	    index_combobox.setBackgroundNonSelectionColor(Gatherer.config.getColor("coloring.editable_background", false));
	    index_combobox.setBackgroundSelectionColor(Gatherer.config.getColor("coloring.collection_selection_background", false));
	    index_combobox.setEditable(true);
	    index_combobox.setTextNonSelectionColor(Gatherer.config.getColor("coloring.workspace_tree_foreground", false));
	    index_combobox.setTextSelectionColor(Gatherer.config.getColor("coloring.collection_selection_foreground", false));

	    JPanel index_button_panel = new JPanel();

	    add_index_button = new JButton("CDM.IndexManager.MGPP.Add_Index");
	    add_index_button.setEnabled(false);
	    add_index_button.setMnemonic(KeyEvent.VK_A);
	    
	    remove_index_button = new JButton("CDM.IndexManager.MGPP.Remove_Index");
	    remove_index_button.setEnabled(false);
	    remove_index_button.setMnemonic(KeyEvent.VK_A);

	    JPanel index_empty_panel = new JPanel();

	    // Connect Indexes
	    EnableAddIndexListener index_eal = new EnableAddIndexListener();
	    add_index_button.addActionListener(new AddIndexActionListener());
	    current_indexes_list.addListSelectionListener(new CurrentIndexesListSelectionListener());
	    Gatherer.dictionary.register(add_index_button, null, false);
	    Gatherer.dictionary.register(current_indexes_label, null, false);
	    Gatherer.dictionary.register(index_label, null, false);
	    Gatherer.dictionary.register(index_name_label, null, false);
	    Gatherer.dictionary.register(move_index_down_label, null, false);
	    Gatherer.dictionary.register(move_index_up_label, null, false);
	    Gatherer.dictionary.register(remove_index_button, null, false);
	    index_combobox.addActionListener(index_eal);
	    ((JTextField)index_combobox.getEditor().getEditorComponent()).getDocument().addDocumentListener(index_eal);
	    index_name_field.getDocument().addDocumentListener(index_eal);
	    move_index_down_button.addActionListener(new MoveIndexDownListener());
	    move_index_up_button.addActionListener(new MoveIndexUpListener());
	    remove_index_button.addActionListener(new RemoveIndexActionListener(index_eal));
	    // Layout Indexes
	    index_movement_panel.setLayout(new GridLayout(2,1));
	    index_movement_panel.add(move_index_up_button);
	    index_movement_panel.add(move_index_down_button);

	    current_indexes_panel.setLayout(new BorderLayout());
	    current_indexes_panel.add(current_indexes_label, BorderLayout.NORTH);
	    current_indexes_panel.add(new JScrollPane(current_indexes_list), BorderLayout.CENTER);
	    current_indexes_panel.add(index_movement_panel, BorderLayout.EAST);

	    index_name_panel.setLayout(new BorderLayout());
	    index_name_panel.add(index_name_label, BorderLayout.WEST);
	    index_name_panel.add(index_name_field, BorderLayout.CENTER);

	    index_panel.setLayout(new BorderLayout());
	    index_panel.add(index_label, BorderLayout.WEST);
	    index_panel.add(index_combobox, BorderLayout.CENTER);

	    index_details_panel.setLayout(new GridLayout(2,1,0,5));
	    index_details_panel.add(index_name_panel);
	    index_details_panel.add(index_panel);

	    index_button_panel.setBorder(BorderFactory.createEmptyBorder(5,0,0,0));
	    index_button_panel.setLayout(new GridLayout(1,2,5,0));
	    index_button_panel.add(add_index_button);
	    index_button_panel.add(remove_index_button);

	    index_body_panel.setBorder(BorderFactory.createEmptyBorder(5,0,0,0));
	    index_body_panel.setLayout(new BorderLayout());
	    index_body_panel.add(index_details_panel, BorderLayout.CENTER);
	    index_body_panel.add(index_button_panel, BorderLayout.SOUTH);

	    index_spacer_panel.setLayout(new BorderLayout());
	    index_spacer_panel.add(index_body_panel, BorderLayout.NORTH);
	    index_spacer_panel.add(index_empty_panel, BorderLayout.CENTER);
	    
	    indexes_panel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
	    indexes_panel.setLayout(new BorderLayout());
	    indexes_panel.add(current_indexes_panel, BorderLayout.NORTH);
	    indexes_panel.add(index_spacer_panel, BorderLayout.CENTER);

	    // Create Levels
	    JPanel levels_panel = new JPanel();
	    
	    JPanel current_levels_panel = new JPanel();

	    current_levels_label = new JLabel("CDM.IndexManager.MGPP.Current_Levels");

	    current_levels_list = new JList(levels_model);
	    current_levels_list.setVisibleRowCount(5);

	    JPanel level_movement_panel = new JPanel();

	    ImageIcon move_level_up_icon = Utility.getImage("arrow-up.gif");
	    move_level_up_button = new JButton();
	    move_level_up_label = new JLabel("CDM.Move.Move_Up");
	    move_level_up_label.setHorizontalAlignment(JLabel.CENTER);
	    move_level_up_label.setPreferredSize(LABEL_SIZE);
	    move_level_up_button.setLayout(new BorderLayout());
	    move_level_up_button.add(new JLabel(move_level_up_icon), BorderLayout.WEST);
	    move_level_up_button.add(move_level_up_label, BorderLayout.CENTER);
	    move_level_up_button.add(new JLabel(move_level_up_icon), BorderLayout.EAST);
	    move_level_up_button.setMnemonic(KeyEvent.VK_U);

	    ImageIcon move_level_down_icon = Utility.getImage("arrow-down.gif");
	    move_level_down_button = new JButton();
	    move_level_down_label = new JLabel("CDM.Move.Move_Down");
	    move_level_down_label.setHorizontalAlignment(JLabel.CENTER);
	    move_level_down_label.setPreferredSize(LABEL_SIZE);
	    move_level_down_button.setLayout(new BorderLayout());
	    move_level_down_button.add(new JLabel(move_level_down_icon), BorderLayout.WEST);
	    move_level_down_button.add(move_level_down_label, BorderLayout.CENTER);
	    move_level_down_button.add(new JLabel(move_level_down_icon), BorderLayout.EAST);
	    move_level_down_button.setMnemonic(KeyEvent.VK_D);

	    JPanel level_spacer_panel = new JPanel();

	    JPanel level_body_panel = new JPanel();

	    JPanel level_details_panel = new JPanel();

	    JPanel level_name_panel = new JPanel();

	    level_name_label = new JLabel("CDM.IndexManager.MGPP.Level_Name");
	    level_name_label.setPreferredSize(LABEL_SIZE);

	    level_name_field = new JTextField();

	    JPanel level_panel = new JPanel();

	    level_label = new JLabel("CDM.IndexManager.MGPP.Level");
	    level_label.setPreferredSize(LABEL_SIZE);

	    level_combobox = new GComboBox(Index.LEVEL);
	    level_combobox.setBackgroundNonSelectionColor(Gatherer.config.getColor("coloring.editable_background", false));
	    level_combobox.setBackgroundSelectionColor(Gatherer.config.getColor("coloring.collection_selection_background", false));
	    level_combobox.setEditable(true);
	    level_combobox.setTextNonSelectionColor(Gatherer.config.getColor("coloring.workspace_tree_foreground", false));
	    level_combobox.setTextSelectionColor(Gatherer.config.getColor("coloring.collection_selection_foreground", false));

	    JPanel level_button_panel = new JPanel();

	    add_level_button = new JButton("CDM.IndexManager.MGPP.Add_Level");
	    add_level_button.setEnabled(false);
	    add_level_button.setMnemonic(KeyEvent.VK_A);
	    
	    remove_level_button = new JButton("CDM.IndexManager.MGPP.Remove_Level");
	    remove_level_button.setEnabled(false);
	    remove_level_button.setMnemonic(KeyEvent.VK_A);

	    JPanel level_empty_panel = new JPanel();

	    // Connect Levels
	    EnableAddLevelListener level_eal = new EnableAddLevelListener();
	    add_level_button.addActionListener(new AddLevelActionListener());
	    current_levels_list.addListSelectionListener(new CurrentLevelsListSelectionListener());
	    Gatherer.dictionary.register(add_level_button, null, false);
	    Gatherer.dictionary.register(current_levels_label, null, false);
	    Gatherer.dictionary.register(level_label, null, false);
	    Gatherer.dictionary.register(level_name_label, null, false);
	    Gatherer.dictionary.register(move_level_down_label, null, false);
	    Gatherer.dictionary.register(move_level_up_label, null, false);
	    Gatherer.dictionary.register(remove_level_button, null, false);
	    level_combobox.addActionListener(level_eal);
	    ((JTextField)level_combobox.getEditor().getEditorComponent()).getDocument().addDocumentListener(level_eal);
	    level_name_field.getDocument().addDocumentListener(level_eal);
	    move_level_down_button.addActionListener(new MoveLevelDownListener());
	    move_level_up_button.addActionListener(new MoveLevelUpListener());
	    remove_level_button.addActionListener(new RemoveLevelActionListener(level_eal));
	    // Layout Levels

	    level_movement_panel.setLayout(new GridLayout(2,1));
	    level_movement_panel.add(move_level_up_button);
	    level_movement_panel.add(move_level_down_button);

	    current_levels_panel.setLayout(new BorderLayout());
	    current_levels_panel.add(current_levels_label, BorderLayout.NORTH);
	    current_levels_panel.add(new JScrollPane(current_levels_list), BorderLayout.CENTER);
	    current_levels_panel.add(level_movement_panel, BorderLayout.EAST);

	    level_name_panel.setLayout(new BorderLayout());
	    level_name_panel.add(level_name_label, BorderLayout.WEST);
	    level_name_panel.add(level_name_field, BorderLayout.CENTER);

	    level_panel.setLayout(new BorderLayout());
	    level_panel.add(level_label, BorderLayout.WEST);
	    level_panel.add(level_combobox, BorderLayout.CENTER);

	    level_details_panel.setLayout(new GridLayout(2,1,0,5));
	    level_details_panel.add(level_name_panel);
	    level_details_panel.add(level_panel);

	    level_button_panel.setBorder(BorderFactory.createEmptyBorder(5,0,0,0));
	    level_button_panel.setLayout(new GridLayout(1,2,5,0));
	    level_button_panel.add(add_level_button);
	    level_button_panel.add(remove_level_button);

	    level_body_panel.setBorder(BorderFactory.createEmptyBorder(5,0,0,0));
	    level_body_panel.setLayout(new BorderLayout());
	    level_body_panel.add(level_details_panel, BorderLayout.CENTER);
	    level_body_panel.add(level_button_panel, BorderLayout.SOUTH);

	    level_spacer_panel.setLayout(new BorderLayout());
	    level_spacer_panel.add(level_body_panel, BorderLayout.NORTH);
	    level_spacer_panel.add(level_empty_panel, BorderLayout.CENTER);
	    
	    levels_panel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
	    levels_panel.setLayout(new BorderLayout());
	    levels_panel.add(current_levels_panel, BorderLayout.NORTH);
	    levels_panel.add(level_spacer_panel, BorderLayout.CENTER);

	    // Create General
	    JPanel header_panel = new JPanel();
	    
	    title_label = new JLabel("CDM.IndexManager.MGPP.Title");

	    instructions_textarea = new JTextArea("CDM.IndexManager.MGPP.Instructions");
	    instructions_textarea.setEditable(false);
	    instructions_textarea.setLineWrap(true);
	    instructions_textarea.setRows(5);
	    instructions_textarea.setWrapStyleWord(true);

	    tabbed_pane = new JTabbedPane();

	    // Layout General
	    header_panel.setBorder(BorderFactory.createEmptyBorder(0,0,2,0));
	    header_panel.setLayout(new BorderLayout());
	    header_panel.add(title_label, BorderLayout.NORTH);
	    header_panel.add(new JScrollPane(instructions_textarea), BorderLayout.CENTER);

	    tabbed_pane.add("CDM.IndexManager.MGPP.Indexes", indexes_panel);
	    tabbed_pane.add("CDM.IndexManager.MGPP.Levels", levels_panel);

	    setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
	    setLayout(new BorderLayout());
	    add(header_panel, BorderLayout.NORTH);
	    add(tabbed_pane, BorderLayout.CENTER);

	    // Connect General
	    Gatherer.dictionary.register(instructions_textarea, null, false);
	    Gatherer.dictionary.register(tabbed_pane, null, false);
	    Gatherer.dictionary.register(title_label, null, false);
	}

	public void destroy() {
	    Gatherer.dictionary.deregister(add_index_button);
	    Gatherer.dictionary.deregister(current_indexes_label);
	    Gatherer.dictionary.deregister(index_label);
	    Gatherer.dictionary.deregister(index_name_label);
	    Gatherer.dictionary.deregister(move_index_down_label);
	    Gatherer.dictionary.deregister(move_index_up_label);
	    Gatherer.dictionary.deregister(remove_index_button);

	    Gatherer.dictionary.deregister(instructions_textarea);
	    Gatherer.dictionary.deregister(tabbed_pane);
	    Gatherer.dictionary.deregister(title_label);
	}

	public void gainFocus() {
	    // Reload the assigned indexes list.
	    index_combobox.removeAllItems();
	    index_combobox.addItem(CollectionConfiguration.ALLFIELDS_STR);
	    index_combobox.addItem(CollectionConfiguration.TEXT_STR);
	    java.util.List assigned_elements = Gatherer.c_man.getCollection().msm.getAssignedElements();
	    int assigned_elements_size = assigned_elements.size();
	    for(int i = 0; i < assigned_elements_size; i++) {
		index_combobox.addItem(assigned_elements.get(i));
	    }
	    assigned_elements = null;
	}

	private class AddIndexActionListener
	    implements ActionListener {
	    public void actionPerformed(ActionEvent event) {
		// Retrieve the name
		String name = index_name_field.getText();
		// Retrieve the source
		Object source = index_combobox.getSelectedItem();
		// If this object isn't yet in the combobox add it.
		if(index_combobox.getSelectedIndex() == -1) {
		    index_combobox.insertItemAt(source, index_combobox.getItemCount());
		}
		// Create new index
		ArrayList sources = new ArrayList();
		sources.add(source);
		Index index = new Index(sources);
		// Create new metadatum
		CollectionMeta metadatum = new CollectionMeta(CollectionConfiguration.STOP_CHARACTER + index.getID());
		metadatum.setValue(name);
		// Assign new index
		addIndex(index, metadatum);
		// Done. Disable add
		add_index_button.setEnabled(false);
	    }
	}

	private class AddLevelActionListener
	    implements ActionListener {
	    public void actionPerformed(ActionEvent event) {
		// Retrieve the name
		String name = level_name_field.getText();
		// Retrieve the source
		String source = (String)level_combobox.getSelectedItem();
		// If this object isn't yet in the combobox add it.
		if(level_combobox.getSelectedIndex() == -1) {
		    level_combobox.insertItemAt(source, level_combobox.getItemCount());
		}
		Level level = new Level(source);
		// Create new metadatum
		CollectionMeta metadatum = new CollectionMeta(CollectionConfiguration.STOP_CHARACTER + source);
		metadatum.setValue(name);
		// Assign new level
		addLevel(level, metadatum);
		// Done. Disable add
		add_level_button.setEnabled(false);
	    }
	}

	private class CurrentIndexesListSelectionListener
	    implements ListSelectionListener {
	    public void valueChanged(ListSelectionEvent event) {
		if(!event.getValueIsAdjusting()) {
		    Index index = (Index)current_indexes_list.getSelectedValue();
		    if(index != null) {
			move_index_down_button.setEnabled((model.indexOf(index) < model.getSize() - 1));
			move_index_up_button.setEnabled((model.indexOf(index) > 0));
			remove_index_button.setEnabled(true);
		    }
		    else {
			move_index_down_button.setEnabled(false);
			move_index_up_button.setEnabled(false);
			remove_index_button.setEnabled(false);
		    }
		}
	    }
	}

	private class CurrentLevelsListSelectionListener
	    implements ListSelectionListener {
	    public void valueChanged(ListSelectionEvent event) {
		if(!event.getValueIsAdjusting()) {
		    Level level = (Level)current_levels_list.getSelectedValue();
		    if(level != null) {
			move_level_down_button.setEnabled((levels_model.indexOf(level) < levels_model.getSize() - 1));
			move_level_up_button.setEnabled((levels_model.indexOf(level) > 0));
			remove_level_button.setEnabled(true);
		    }
		    else {
			move_level_down_button.setEnabled(false);
			move_level_up_button.setEnabled(false);
			remove_level_button.setEnabled(false);
		    }
		}
	    }
	}

	private class EnableAddIndexListener
	    implements ActionListener, DocumentListener {
	    /** Called whenever a selection action occurs on the combobox.
	     * @param  event an ActionEvent containing information about the selection event
	     */
	    public void actionPerformed(ActionEvent event) {
		validateAddButton();
	    }

	    /** Gives notification that an attribute or set of attributes changed.
	     * @param  event a DocumentEvent containing information about the text changed
	     */
	    public void changedUpdate(DocumentEvent event) {
		validateAddButton();
	    }

	    /** Gives notification that there was an insert into the document.
	     * @param  event a DocumentEvent containing information about the text added
	     */
	    public void insertUpdate(DocumentEvent event) {
		validateAddButton();
	    }
          
	    /** Gives notification that a portion of the document has been removed.
	     * @param  event a DocumentEvent containing information about the text removed
	     */
	    public void removeUpdate(DocumentEvent e) {
		validateAddButton();
	    }
	    
	    /** Change the enable state of the add button depending on the current value in the search type combobox. */
	    public void validateAddButton() {
		String name = index_name_field.getText();
		Object selected_object = index_combobox.getSelectedItem();
		if(name.length() > 0 && selected_object != null) {
		    // Unfortunately we have to generate a valid id
		    String id = null;
		    if(selected_object instanceof ElementWrapper) {
			id = ((ElementWrapper)selected_object).getName();
		    }
		    else {
			id = selected_object.toString();
		    }
		    add_index_button.setEnabled(getIndex(id) == null);
		}
		else {
		    add_index_button.setEnabled(false);
		}
	    }
	}

	private class EnableAddLevelListener
	    implements ActionListener, DocumentListener {
	    /** Called whenever a selection action occurs on the combobox.
	     * @param  event an ActionEvent containing information about the selection event
	     */
	    public void actionPerformed(ActionEvent event) {
		validateAddButton();
	    }

	    /** Gives notification that an attribute or set of attributes changed.
	     * @param  event a DocumentEvent containing information about the text changed
	     */
	    public void changedUpdate(DocumentEvent event) {
		validateAddButton();
	    }

	    /** Gives notification that there was an insert into the document.
	     * @param  event a DocumentEvent containing information about the text added
	     */
	    public void insertUpdate(DocumentEvent event) {
		validateAddButton();
	    }
          
	    /** Gives notification that a portion of the document has been removed.
	     * @param  event a DocumentEvent containing information about the text removed
	     */
	    public void removeUpdate(DocumentEvent e) {
		validateAddButton();
	    }
	    
	    /** Change the enable state of the add button depending on the current value in the search type combobox. */
	    public void validateAddButton() {
		String name = level_name_field.getText();
		Object selected_object = level_combobox.getSelectedItem();
		if(name.length() > 0 && selected_object != null) {
		    add_level_button.setEnabled(getLevel((String)selected_object) == null);
		}
		else {
		    add_level_button.setEnabled(false);
		}
	    }
	}

	private class MoveIndexDownListener
	    implements ActionListener {
	    public void actionPerformed(ActionEvent event) {
		// Retrieve the first selected item
		Index index = (Index) current_indexes_list.getSelectedValue();
		moveIndex(index, false);
		current_indexes_list.setSelectedValue(index, true);
	    }
	}

	private class MoveLevelDownListener
	    implements ActionListener {
	    public void actionPerformed(ActionEvent event) {
		// Retrieve the first selected item
		Level level = (Level) current_levels_list.getSelectedValue();
		moveLevel(level, false);
		current_levels_list.setSelectedValue(level, true);
	    }
	}

	private class MoveIndexUpListener
	    implements ActionListener {
	    public void actionPerformed(ActionEvent event) {
		// Retrieve the first selected item
		Index index = (Index) current_indexes_list.getSelectedValue();
		moveIndex(index, true);
		current_indexes_list.setSelectedValue(index, true);
	    }
	}

	private class MoveLevelUpListener
	    implements ActionListener {
	    public void actionPerformed(ActionEvent event) {
		// Retrieve the first selected item
		Level level = (Level) current_levels_list.getSelectedValue();
		moveLevel(level, true);
		current_levels_list.setSelectedValue(level, true);
	    }
	}

	private class RemoveIndexActionListener
	    implements ActionListener {

	    private EnableAddIndexListener eal = null;

	    public RemoveIndexActionListener(EnableAddIndexListener eal) {
		this.eal = eal;
	    }

	    public void actionPerformed(ActionEvent event) {
		// Retrieve the selected items
		Object[] selected_objects = current_indexes_list.getSelectedValues();
		// Clear selection
		current_indexes_list.clearSelection();
		for(int i = 0; i < selected_objects.length; i++) {
		    Index index = (Index) selected_objects[i];
		    // Remove any related metadata
		    CollectionDesignManager.collectionmeta_manager.removeMetadata(CollectionConfiguration.STOP_CHARACTER + index.getID());
		    // Remove the index
		    removeIndex(index);
		}
		// Disable remove button
		remove_index_button.setEnabled(false);
		// Check if add should be reenabled
		eal.validateAddButton();
	    }
	}

	private class RemoveLevelActionListener
	    implements ActionListener {

	    private EnableAddLevelListener eal = null;

	    public RemoveLevelActionListener(EnableAddLevelListener eal) {
		this.eal = eal;
	    }

	    public void actionPerformed(ActionEvent event) {
		// Retrieve the selected items
		Object[] selected_objects = current_levels_list.getSelectedValues();
		// Clear selection
		current_levels_list.clearSelection();
		for(int i = 0; i < selected_objects.length; i++) {
		    Level level = (Level) selected_objects[i];
		    // Remove any related metadata
		    CollectionDesignManager.collectionmeta_manager.removeMetadata(CollectionConfiguration.STOP_CHARACTER + level.getName());
		    // Remove the index
		    removeLevel(level);
		}
		// Disable remove button
		remove_level_button.setEnabled(false);
		// Check if add should be reenabled
		eal.validateAddButton();
	    }
	}
    }
}


