/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */
package org.greenstone.gatherer.cdm;
import java.awt.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import org.greenstone.gatherer.Gatherer;
import org.greenstone.gatherer.cdm.CollectionDesignManager;
import org.greenstone.gatherer.cdm.Control;
import org.greenstone.gatherer.cdm.ElementWrapper;
import org.greenstone.gatherer.msm.MetadataSet;
import org.greenstone.gatherer.msm.MSMEvent;
import org.greenstone.gatherer.msm.MSMUtils;
import org.greenstone.gatherer.msm.MSMListener;
import org.w3c.dom.*;
/** Unlike its namesake in the msm package, this class really only knows how to produce a simple visual representation of the currently imported metadata sets. It is also read-only, so should be fairly straight forward. (It has no namesake any longer as my co-programmers found this far too confusing - however I have now added a CollectionConfiguration to the cdm package just to keep them on their toes!)
 * @author John Thompson, Greenstone Digital Library, University of Waikato
 * @version 2.3d
 * @see org.greenstone.gatherer.msm.MetadataSetManager
 */
public class MetadataSetView
    extends DynamicListModel
    implements MSMListener {
    /** The visual contols used to review the metadata sets. */
    private Control controls = null;
    /** A reference to ourselves so our inner classes can refer to us. */
    private DynamicListModel model = null;
    /** Constructor.
     */
    public MetadataSetView() {
	Gatherer.println("MetadataSetView: Initialized.");
	model = this;
	Gatherer.c_man.getCollection().msm.addMSMListener(this);
	loadMetadataSets();
	// Build the controls
	controls = new MetadataSetControl();
    }
    /** Destructor. Remove any references of this class from persistant objects.
		* @see org.greenstone.gatherer.Gatherer
	  * @see org.greenstone.gatherer.collection.CollectionManager
	  * @see org.greenstone.gatherer.msm.MetadataSetManager
	  */
    public void destroy() {
	controls.destroy();
	controls = null;
	Gatherer.c_man.msm.removeMSMListener(this);
	model = null;
    }
    /** Called when an element is changed within a set in the MSM, prompting us to refresh our list of elements being shown.
     * @param event A <strong>MSMEvent</strong> which encapsulates relevant data about the change.
     * @see org.greenstone.gatherer.cdm.MetadataSetManager.Control
     */
    public void elementChanged(MSMEvent event) {
	// Get the controls to refresh element list.
	((MetadataSetControl)controls).refreshElementList();
    }
    /** A method for retrieve the controls for this manager.
     * @see org.greenstone.gatherer.Dictionary
     * @see org.greenstone.gatherer.gui.Coloring
     * @see org.greenstone.gatherer.msm.MetadataSetManager
     */
    public Control getControls() {
	return controls;
    }
    /** Called when a metadata value has undergone significant change.
     * @param event A <strong>MSMEvent</strong> which encapsulates relevant data about the change.
     */
    public void metadataChanged(MSMEvent event) {
	// Couldn't care less.
    }
    /** Called when a set is added or removed from the MSM.
     * @param event A <strong>MSMEvent</strong> which encapsulates relevant data about the change.
     */
    public void setChanged(MSMEvent event) {
	// Reload model.
	clear();
	loadMetadataSets();
    }
    /** Select the selected element, given its name, and return the bounds of the selection. Used during search and replace.
     * @param element The elements fully qualified name as a <strong>String</strong>.
     * @return The bounds of the selection as a <strong>Rectangle</strong>.
     * @see org.greenstone.gatherer.cdm.MetadataSetManager.Control
     */
    public Rectangle setSelectedElement(String element) {
	return ((MetadataSetControl)controls).setSelectedElement(element);
    }
    /** Prints out the contents of this manager, as they would appear in the collection configuration file.
     * @return A <strong>String</strong> containing a block of commands.
     * @see org.greenstone.gatherer.cdm.MetadataSetManager.SetWrapper
     */
    public String toString() {
	String text = "";
	for(int i = 0; i < size(); i++) {
	    SetWrapper set = (SetWrapper)get(i);
	    text = text + set.toString() + "\n";
	}
	text = text + "\n";
	return text;
    }
    /** Called when a significant change has occured to a value tree for a certain element, however we take no further action.
     * @param event A <strong>MSMEvent</strong> containing information relevant to the event.
     */
    public void valueChanged(MSMEvent event) {
    }
    /** Method to retrieve a phrase from the dictionary based on a key.
     * @param key A <strong>String</strong> used to find the correct phrase.
     * @param args A <strong>String[]</strong> of arguments used in formatting and filling out the phrase.
     * @return A <strong>String</strong> containing the correct phrase with the correct formatting.
     */
    private String get(String key) {
	return get(key, null);
    }
    /** Method to retrieve a phrase from the dictionary based on a key.
     * @param key A <strong>String</strong> used to find the correct phrase.
     * @return A <strong>String</strong> containing the correct phrase with the correct formatting.
     * @see org.greenstone.gatherer.Dictionary
     * @see org.greenstone.gatherer.Gatherer
     */
    private String get(String key, String args[]) {
	if(key.indexOf(".") == -1) {
	    key = "CDM.MetadataSetManager." + key;
	}
	return Gatherer.dictionary.get(key, args);
    }
    /** Retrieves the current list of loaded metadata sets, and builds a list model around them.
     * @see org.greenstone.gatherer.Gatherer
     * @see org.greenstone.gatherer.cdm.MetadataSetManager.SetWrapper
     * @see org.greenstone.gatherer.collection.CollectionManager
     * @see org.greenstone.gatherer.msm.MetadataSet
     * @see org.greenstone.gatherer.msm.MetadataSetManager
     */
    private void loadMetadataSets() {
	// We initialize the set_model with wrapped metadata sets. Note that when we call getSets() we also end up adding ourselves as a listener to the metadata set manager.
	Vector sets = Gatherer.c_man.getCollection().msm.getSets();
	for(int i = 0; i < sets.size(); i++) {
	    addElement(new SetWrapper((MetadataSet)sets.get(i)));
	}
    }
    /** This class creates and lays-out the various controls for reviewing the metadata sets, and their commands as they would appear in the collection configuration file. */
    private class MetadataSetControl
	extends JPanel 
	implements Control {
	/** The label denoting the element list. */
	private JLabel element_label = null;
	/** The label denoting the set list. */
	private JLabel set_label = null;
	/** The title of these controls. */
	private JLabel title = null;
	/** The list of elements for the choosen set. */
	private JList element_list = null;
	/** The list of sets in this collection. */
	private JList set_list = null;
	/** The panel onto which all other panels will be placed. */
	private JPanel central_pane = null;
	/** The panel onto which the element list will be placed. */
	private JPanel element_pane = null;
	/** The panel containing the title and instructions. */
	private JPanel header_pane = null;
	/** The panel containing the set list. */
	private JPanel set_pane = null;
	/** The text area of inline instructions. */
	private JTextArea instructions = null;
	/** The element model for the currently selected set. */
	private Vector element_model = null;
	/* Constructor.
	 * @see org.greenstone.gatherer.Coloring;
	 * @see org.greenstone.gatherer.Dictionary
	 * @see org.greenstone.gatherer.cdm.MetadataSetManager.ListListener
	 * @see org.greenstone.gatherer.msm.MetadataSetManager
	 */
	public MetadataSetControl() {
				// Create visual components
	    central_pane = new JPanel();
	    element_label = new JLabel(get("Elements"));
	    element_label.setHorizontalAlignment(JLabel.CENTER);
	    element_label.setOpaque(true);
	    element_list = new JList();
	    element_list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	    element_pane = new JPanel();
	    header_pane = new JPanel();
	    instructions = new JTextArea(get("Instructions"));
	    instructions.setBackground(Gatherer.config.getColor("coloring.collection_tree_background", false));
	    instructions.setEditable(false);
	    instructions.setLineWrap(true);
	    instructions.setRows(5);
	    instructions.setWrapStyleWord(true);
	    set_label = new JLabel(get("Sets"));
	    set_label.setHorizontalAlignment(JLabel.CENTER);
	    set_label.setOpaque(true);
	    set_list = new JList(model);
	    set_list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	    set_pane = new JPanel();
	    title = new JLabel(get("Title"));
	    title.setHorizontalAlignment(JLabel.CENTER);
	    title.setOpaque(true);
				// Add listeners
	    set_list.addListSelectionListener(new ListListener());
				// Layout
	    instructions.setBorder(BorderFactory.createEmptyBorder(2,5,2,5));

	    header_pane.setLayout(new BorderLayout());
	    header_pane.add(title, BorderLayout.NORTH);
	    header_pane.add(new JScrollPane(instructions), BorderLayout.CENTER);

	    set_pane.setLayout(new BorderLayout());
	    set_pane.add(set_label, BorderLayout.NORTH);
	    set_pane.add(new JScrollPane(set_list), BorderLayout.CENTER);

	    element_pane.setLayout(new BorderLayout());
	    element_pane.add(element_label, BorderLayout.NORTH);
	    element_pane.add(new JScrollPane(element_list), BorderLayout.CENTER);

	    central_pane.setLayout(new GridLayout(2,1));
	    central_pane.add(set_pane);
	    central_pane.add(element_pane);

	    setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
	    setLayout(new BorderLayout());
	    add(header_pane, BorderLayout.NORTH);
	    add(central_pane, BorderLayout.CENTER);
	}
	/** Destructor. */
	public void destroy() {
	}
	/** Update the element list. */
	public void refreshElementList() {
	    element_list.updateUI();
	}
	/** Select the selected element, given its name, and return the bounds of the selection. Used during search and replace.
		* @param element The elements fully qualified name as a <strong>String</strong>.
		* @return The bounds of the selection as a <strong>Rectangle</strong>.
		* @see org.greenstone.gatherer.cdm.MetadataSetManager.Control
		* @see org.greenstone.gatherer.cdm.MetadataSetManager.Control.ElementWrapper
		*/
	public Rectangle setSelectedElement(String element) {
	    Rectangle bounds = null;
				// Parse off namespace
	    String namespace = element.substring(0, element.indexOf("."));
				// Force the set list to show the correct entry.
	    ListModel s_model = set_list.getModel();
	    for(int i = 0; i < s_model.getSize(); i++) {
		SetWrapper sw = (SetWrapper) s_model.getElementAt(i);
		if(sw.getSet().getNamespace().equals(namespace)) {
		    set_list.setSelectedValue(sw, true);
		}
	    }
				// Force the element list to show that name.
	    ListModel e_model = element_list.getModel();
	    for(int i = 0; i < e_model.getSize(); i++) {
		ElementWrapper ew = (ElementWrapper) e_model.getElementAt(i);
		if(ew.name().equals(element)) {
		    element_list.setSelectedValue(ew, true);
		    bounds = element_list.getCellBounds(i, i);
		}
	    }
				// Return the bounds of the selected row.
	    return bounds;
	}
	/** Overriden to ensure the instruction area is scrolled to top.
		  */
	public void gainFocus() {
	    if(instructions != null) {
		instructions.setCaretPosition(0);
	    }
	}

	public void loseFocus() {
	}

	private class ListListener
	    implements ListSelectionListener {
	    public void valueChanged(ListSelectionEvent event) {
		if(!set_list.isSelectionEmpty()) {
		    MetadataSet set = ((SetWrapper)set_list.getSelectedValue()).getSet();
		    NodeList elements = set.getElements();
		    element_model = new Vector();
		    for(int i = 0; i < elements.getLength(); i++) {
			element_model.add(new ElementWrapper(elements.item(i)));
		    }
		    Collections.sort(element_model);
		    element_list.setListData(element_model);
		}
	    }
	}
    }
    /** Provides a convience wrapper around a metadata set, that allows it to appear in the <strong>JList</strong> the same way it would in the collection configuration file. */
    private class SetWrapper {
	private MetadataSet set = null;
	public SetWrapper(MetadataSet set) {
	    this.set = set;
	}
	public MetadataSet getSet() {
	    return set;
	}
	public String toString() {
	    String namespace = set.getNamespace();
				// Watch out for the greenstone set, with its namespace of ""
	    if(namespace == null || namespace.length() == 0) {
		namespace = "greenstone";
	    }
	    return "metadataset " + namespace + " \"" + set.getFile().getName() + "\"";
	}
    }
}
