/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */
package org.greenstone.gatherer.cdm;
/**************************************************************************************
 * Written:      16/07/03
 * Revised:      
 **************************************************************************************/
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import org.greenstone.gatherer.Gatherer;
import org.greenstone.gatherer.cdm.CollectionConfiguration;
import org.greenstone.gatherer.cdm.CollectionDesignManager;
import org.greenstone.gatherer.cdm.Control;
import org.greenstone.gatherer.cdm.SearchType;
import org.greenstone.gatherer.cdm.DOMProxyListModel;
import org.greenstone.gatherer.gui.GComboBox;
import org.greenstone.gatherer.msm.MSMUtils;
import org.greenstone.gatherer.util.Utility;
import org.w3c.dom.*;
/** This class maintains an ordered list of the search types available in the collection (MGPP command available in G2.39 or later). Currently only 'form' and 'plain' are valid.
 * @author John Thompson, Greenstone Digital Library, University of Waikato
 * @version 2.4
 */
public class SearchTypeManager
    extends DOMProxyListModel {

    /** The default size of a label, in an attempt to make labels/fields line up. */
    static final private Dimension LABEL_SIZE = new Dimension(150,25);

    static final private String[] SEARCH_TYPES = {"form", "plain"};

    /** The controls used to edit the search types. */
    private Control controls = null;
    /** A reference to ourselves so our inner classes have access. */
    private DOMProxyListModel model;

    public SearchTypeManager(Element searchtypes_element) {
	super(searchtypes_element, CollectionConfiguration.CONTENT_ELEMENT, new SearchType());
	this.model = this;
	Gatherer.println("SearchTypeManager: parsed " + getSize() + " search types.");
    }

    public void addSearchType(SearchType searchtype) {
	if(!contains(searchtype)) {
	    add(getSize(), searchtype);
	    Gatherer.c_man.configurationChanged();
	}
    }

    public Control getControls() {
	if(controls == null) {
	    controls = new SearchTypeControl();
	}
	return controls;
    }

    /** Be examining the SearchType 'root' we were created with, determine if mgpp is enabled.
     * @return true if MGPP is enabled, false otherwise
     */
    public boolean isMGPPEnabled() {
	return root.getAttribute(CollectionConfiguration.ASSIGNED_ATTRIBUTE).equals(CollectionConfiguration.TRUE_STR);
    }

    public void removeSearchType(SearchType searchtype) {
	if(contains(searchtype)) {
	    remove(searchtype);
	    Gatherer.c_man.configurationChanged();
	}
    }

    private class SearchTypeControl
	extends JPanel
	implements Control {

	private GComboBox search_type_combobox;

	private JButton add_button;
	private JButton move_down_button;
	private JButton move_up_button;
	private JButton remove_button;
	
	private JCheckBox enable_advanced_searches_checkbox;

	private JLabel current_search_types_label;
	private JLabel move_down_label;
	private JLabel move_up_label;
	private JLabel search_type_label;
	private JLabel title_label;

	private JList current_search_types_list;

	private JTextArea instructions_textarea;

	public SearchTypeControl() {
	    // Creation
	    JPanel instructions_panel = new JPanel();
	    title_label = new JLabel("CDM.SearchTypeManager.Title");
	    title_label.setHorizontalAlignment(JLabel.CENTER);
	    instructions_textarea = new JTextArea("CDM.SearchTypeManager.Instructions");
	    instructions_textarea.setCaretPosition(0);
	    instructions_textarea.setEditable(false);
	    instructions_textarea.setLineWrap(true);
	    instructions_textarea.setRows(4);
	    instructions_textarea.setWrapStyleWord(true);

	    JPanel spacer_panel = new JPanel();

	    JPanel empty_panel = new JPanel();

	    JPanel inner_panel = new JPanel();
	    
	    enable_advanced_searches_checkbox = new JCheckBox("CDM.SearchTypeManager.Enable");

	    JPanel current_search_types_panel = new JPanel();
	    current_search_types_label = new JLabel("CDM.SearchTypeManager.Assigned");
	    current_search_types_list = new JList(model);
	    current_search_types_list.setVisibleRowCount(3);

	    JPanel movement_panel = new JPanel();
	    move_up_button = new JButton();
	    move_up_label = new JLabel("CDM.Move.Move_Up");
	    move_up_label.setHorizontalAlignment(JLabel.CENTER);
	    move_up_label.setPreferredSize(LABEL_SIZE);
	    move_up_button.setLayout(new BorderLayout());
	    ImageIcon move_up_icon = Utility.getImage("arrow-up.gif");
	    move_up_button.add(new JLabel(move_up_icon), BorderLayout.WEST);
	    move_up_button.add(move_up_label, BorderLayout.CENTER);
	    move_up_button.add(new JLabel(move_up_icon), BorderLayout.EAST);
	    move_up_button.setMnemonic(KeyEvent.VK_U);
	    move_down_button = new JButton();
	    move_down_label = new JLabel("CDM.Move.Move_Down");
	    move_down_label.setHorizontalAlignment(JLabel.CENTER);
	    move_down_label.setPreferredSize(LABEL_SIZE);
	    move_down_button.setLayout(new BorderLayout());
	    ImageIcon move_down_icon = Utility.getImage("arrow-down.gif");
	    move_down_button.add(new JLabel(move_down_icon), BorderLayout.WEST);
	    move_down_button.add(move_down_label, BorderLayout.CENTER);
	    move_down_button.add(new JLabel(move_down_icon), BorderLayout.EAST);
	    move_down_button.setMnemonic(KeyEvent.VK_D);

	    JPanel search_type_panel = new JPanel();
	    search_type_label = new JLabel("CDM.SearchTypeManager.SearchType");
	    search_type_combobox = new GComboBox(SEARCH_TYPES);
	    search_type_combobox.setBackgroundNonSelectionColor(Gatherer.config.getColor("coloring.editable_background", false));
	    search_type_combobox.setBackgroundSelectionColor(Gatherer.config.getColor("coloring.collection_selection_background", false));
	    search_type_combobox.setEditable(true);
	    search_type_combobox.setSelectedIndex(0);
	    search_type_combobox.setTextNonSelectionColor(Gatherer.config.getColor("coloring.workspace_tree_foreground", false));
	    search_type_combobox.setTextSelectionColor(Gatherer.config.getColor("coloring.collection_selection_foreground", false));

	    JPanel button_panel = new JPanel();
	    add_button = new JButton("CDM.SearchTypeManager.Add");
	    add_button.setEnabled(false);
	    add_button.setMnemonic(KeyEvent.VK_A);

	    remove_button = new JButton("CDM.SearchTypeManager.Remove");
	    remove_button.setEnabled(false);
	    remove_button.setMnemonic(KeyEvent.VK_R);

	    // Connection
	    add_button.addActionListener(new AddActionListener());
	    current_search_types_list.addListSelectionListener(new CurrentSearchTypesListSelectionListener());
	    enable_advanced_searches_checkbox.addActionListener(new EnableAdvancedSearchesActionListener());
	    Gatherer.dictionary.register(add_button, null, false);
	    Gatherer.dictionary.register(current_search_types_label, null, false);
	    Gatherer.dictionary.register(enable_advanced_searches_checkbox, null, false);
	    Gatherer.dictionary.register(instructions_textarea, null, false);
	    Gatherer.dictionary.register(move_down_label, null, false);
	    Gatherer.dictionary.register(move_up_label, null, false);
	    Gatherer.dictionary.register(remove_button, null, false);
	    Gatherer.dictionary.register(search_type_label, null, false);
	    Gatherer.dictionary.register(title_label, null, false);
	    remove_button.addActionListener(new RemoveActionListener());
	    SearchTypesActionDocumentListener stadl = new SearchTypesActionDocumentListener();
	    search_type_combobox.addActionListener(stadl);
	    ((JTextField)search_type_combobox.getEditor().getEditorComponent()).getDocument().addDocumentListener(stadl);
	    // Layout
	    instructions_panel.setBorder(BorderFactory.createEmptyBorder(0,0,2,0));
	    instructions_panel.setLayout(new BorderLayout());
	    instructions_panel.add(title_label, BorderLayout.NORTH);
	    instructions_panel.add(new JScrollPane(instructions_textarea), BorderLayout.CENTER);

	    movement_panel.setBorder(BorderFactory.createEmptyBorder(0,2,0,0));
	    movement_panel.setLayout(new GridLayout(2,1,5,0));
	    movement_panel.add(move_up_button);
	    movement_panel.add(move_down_button);

	    current_search_types_panel.setBorder(BorderFactory.createEmptyBorder(2,0,2,0));
	    current_search_types_panel.setLayout(new BorderLayout());
	    current_search_types_panel.add(current_search_types_label, BorderLayout.NORTH);
	    current_search_types_panel.add(new JScrollPane(current_search_types_list), BorderLayout.CENTER);
	    current_search_types_panel.add(movement_panel, BorderLayout.EAST);

	    button_panel.setBorder(BorderFactory.createEmptyBorder(2,0,0,0));
	    button_panel.setLayout(new GridLayout(1,2,0,5));
	    button_panel.add(add_button);
	    button_panel.add(remove_button);

	    search_type_panel.setLayout(new BorderLayout());
	    search_type_panel.add(search_type_label, BorderLayout.WEST);
	    search_type_panel.add(search_type_combobox, BorderLayout.CENTER);
	    search_type_panel.add(button_panel, BorderLayout.SOUTH);

	    inner_panel.setLayout(new BorderLayout());
	    inner_panel.add(enable_advanced_searches_checkbox, BorderLayout.NORTH);
	    inner_panel.add(current_search_types_panel, BorderLayout.CENTER);
	    inner_panel.add(search_type_panel, BorderLayout.SOUTH);
	    
	    spacer_panel.setLayout(new BorderLayout());
	    spacer_panel.add(inner_panel, BorderLayout.NORTH);
	    spacer_panel.add(empty_panel, BorderLayout.CENTER);

	    setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
	    setLayout(new BorderLayout());
	    add(instructions_panel, BorderLayout.NORTH);
	    add(spacer_panel, BorderLayout.CENTER);
	}

	public void destroy() {
	    Gatherer.dictionary.deregister(add_button);
	    Gatherer.dictionary.deregister(current_search_types_label);
	    Gatherer.dictionary.deregister(enable_advanced_searches_checkbox);
	    Gatherer.dictionary.deregister(instructions_textarea);
	    Gatherer.dictionary.deregister(move_down_label);
	    Gatherer.dictionary.deregister(move_up_label);
	    Gatherer.dictionary.deregister(remove_button);
	    Gatherer.dictionary.deregister(search_type_label);
	    Gatherer.dictionary.deregister(title_label);
	}
	
	public void gainFocus() {
	    validateControls(isMGPPEnabled());
	}

	public void loseFocus() {

	}

	private void validateControls(boolean advanced_search_enabled) {
	    // Enable or disable controls based on whether MGPP is enabled
	    add_button.setEnabled(add_button.isEnabled() && advanced_search_enabled);
	    current_search_types_list.setEnabled(advanced_search_enabled);
	    enable_advanced_searches_checkbox.setSelected(advanced_search_enabled);
	    search_type_combobox.setEnabled(advanced_search_enabled);
	    remove_button.setEnabled(remove_button.isEnabled() && advanced_search_enabled);
	}

	/** Listenes for actions on the Add button, and if detected adds a new search type. */
	private class AddActionListener
	    implements ActionListener {
	    /** Called when someone actions the Add button.
	     * @param  event an ActionEvent containing information about the add button click
	     */
	    public void actionPerformed(ActionEvent event) {
		Object selected_item = search_type_combobox.getSelectedItem();
		if(selected_item != null) {
		    if(search_type_combobox.getSelectedIndex() == -1) {
			search_type_combobox.insertItemAt(selected_item, search_type_combobox.getItemCount());
		    }
		    // Add the search type
		    SearchType new_searchtype = new SearchType((String)selected_item);
		    addSearchType(new_searchtype);
		}
		add_button.setEnabled(false);
	    }
	}

	/** Listens for selections within the search types list and updates the remove button appropriately. */
	private class CurrentSearchTypesListSelectionListener
	    implements ListSelectionListener {
	    /** Called when the selection in the list changes.
	     * @param  event a ListSelectionEvent containing information about the selection change
	     */
	    public void valueChanged(ListSelectionEvent event) {
		if(!event.getValueIsAdjusting()) {
		    remove_button.setEnabled(!current_search_types_list.isSelectionEmpty());
		}
	    }
	}

	/** The most complex listener in this class, this listens for changes to the enable advanced searches checkbox, and when they occur it not only updates the controls on this page, but asks the IndexManager to action the appropriate changes to the underlying DOM so as to support either MG or MGPP styles of indexes. */
	private class EnableAdvancedSearchesActionListener
	    implements ActionListener {
	    /** Called whenever the checkbox is checked or unchecked.
	     * @param  event an ActionEvent containing information about the checking action
	     */
	    public void actionPerformed(ActionEvent event) {
		Gatherer.g_man.wait(true);
		boolean advanced_search_enabled = enable_advanced_searches_checkbox.isSelected();
		model.root.setAttribute(CollectionConfiguration.ASSIGNED_ATTRIBUTE, (advanced_search_enabled ? CollectionConfiguration.TRUE_STR : CollectionConfiguration.FALSE_STR));
		validateControls(advanced_search_enabled);
		CollectionDesignManager.index_manager.setMGPPEnabled(advanced_search_enabled);
		Gatherer.g_man.wait(false);
	    }
	}

	/** Listenes for actions on the Remove button, and if detected removes the currently selected search types. */
	private class RemoveActionListener
	    implements ActionListener {
	    /** Called when someone actions the Remove button.
	     * @param  event an ActionEvent containing information about the remove button click
	     */
	    public void actionPerformed(ActionEvent event) {
		if(!current_search_types_list.isSelectionEmpty()) {
		    Object[] selected_items = current_search_types_list.getSelectedValues();
		    for(int i = 0; i < selected_items.length; i++) {
			removeSearchType((SearchType)selected_items[i]);
		    }
		}
		remove_button.setEnabled(false);
	    }
	}

	/** Listens for changes in the search types combobox, and enabled add button appropriately. */
	private class SearchTypesActionDocumentListener 
	    implements ActionListener, DocumentListener {
	    /** Called whenever a selection action occurs on the combobox.
	     * @param  event an ActionEvent containing information about the selection event
	     */
	    public void actionPerformed(ActionEvent event) {
		validateAddButton();
	    }

	    /** Gives notification that an attribute or set of attributes changed.
	     * @param  event a DocumentEvent containing information about the text changed
	     */
	    public void changedUpdate(DocumentEvent event) {
		validateAddButton();
	    }
	    /** Gives notification that there was an insert into the document.
	     * @param  event a DocumentEvent containing information about the text added
	     */
	    public void insertUpdate(DocumentEvent event) {
		validateAddButton();
	    }
          
	    /** Gives notification that a portion of the document has been removed.
	     * @param  event a DocumentEvent containing information about the text removed
	     */
	    public void removeUpdate(DocumentEvent e) {
		validateAddButton();
	    }
	    
	    /** Change the enable state of the add button depending on the current value in the search type combobox. */
	    private void validateAddButton() {
		Object selected_object = search_type_combobox.getSelectedItem();
		if(selected_object != null) {
		    add_button.setEnabled(model.contains(selected_object));
		}
		else {
		    add_button.setEnabled(false);
		}
	    }
	}
    }
}
