/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */
package org.greenstone.gatherer.util;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridLayout;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.util.StringTokenizer;
import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import org.greenstone.gatherer.Gatherer;
/** This class wraps an email address, separating address and host, and providing a pretty printer. 
 * @author John Thompson, Greenstone Digital Library, University of Waikato
 * @version 2.3d
 */
public class EmailAddress {
    /** The controls for editing this email. */
    private Control controls = null;
    /** A reference to the Gatherer. */
    private Gatherer gatherer = null;
    /** The type of this email, in collection configuration terms. */
    private int type = -1;
    /** The host part of the email (after @). */
    public String host = null;
    /** The user part of the email (before @). */
    public String user = null;
    /** The name given to or associated with this particular email (maybe the persons name, or it this email belongs to a certain variable i.e Collection Author). */
    public String name = null;
    /** Enumeration of important email address types - in this case the creator of a collection. */
    static final public int CREATOR = 0;
    /** Enumeration of important email address types - in this case the maintainer of a collection. */
    static final public int MAINTAINER = 1;
    /** Enumeration of important email address types - in this case any other email address other than the two noted above. */
    static final public int OTHER = 2;
    /** Constructor.
     * @param gatherer A reference to the <strong>Gatherer</strong> for messaging purposes.
     */
    public EmailAddress(Gatherer gatherer) {
	this.gatherer = gatherer;
    }
    /** Constructor.
     * @param gatherer A reference to the <strong>Gatherer</strong> for messaging purposes.
     * @param raw A <strong>String</strong> containing both the name and email separated by whitespace.
     */
    public EmailAddress(Gatherer gatherer, String raw) {
	this(gatherer);
	StringTokenizer tokenizer = new StringTokenizer(raw);
	if(tokenizer.countTokens() >= 2) {
	    this.name = tokenizer.nextToken();
	    if(name.equals("creator")) {
		type = CREATOR;
	    }
	    else if(name.equals("maintainer")) {
		type = MAINTAINER;
	    }
	    else {
		type = OTHER;
	    }
	    String email = tokenizer.nextToken();
	    if(email.indexOf("@") != -1) {
		this.user = email.substring(0, email.indexOf("@"));
		this.host = email.substring(email.indexOf("@") + 1);
	    }
	}
	else {
	    this.name = "Invalid Email";
	    this.user = "";
	    this.host = "";
	}
    }
    /** Retrieve the controls associated with editing this email address.
     * @return A <strong>JPanel</strong> containing those controls.
     */
    public JPanel getControls() {
	if(controls == null) {
	    controls = new Control(name);
	}
	return controls;
    }
    /** Retrieve the controls associated with editing this email address, but ensure they have a certain title label.
     * @param label The <strong>String</strong> to use as the label.
     * @return A <strong>JPanel</strong> containing those controls.
     */
    public JPanel getControls(String label) {
	if(controls == null) {
	    controls = new Control(label);
	}
	return controls;
    }
    /** Method to produce this email as a string.
     * @return A <strong>String</string> representing this email.
     */
    public String toString() {
	return name + " " + user + "@" + host;
    }
    /** Determines if this is a valid email. 
     * @return <i>true</i> if it is current valid, <i>false</i> otherwise. 
     */
    public boolean isValid() {
	return (user != null && user.length() > 0) && (host != null && host.length() > 0);
    }
    /** Overloaded to call get with both a key and an empty argument array.
     * @param key A <strong>String</strong> which is mapped to a initial String within the ResourceBundle.
     * @return A <strong>String</strong> which has been referenced by the key String and that either contains no argument fields, or has had the argument fields automatiically populated with formatting Strings of with argument String provided in the get call.
     */
    private String get(String key) {
	return get(key, null);
    }
    /** Used to retrieve a property value from the Locale specific ResourceBundle, based upon the key and arguments supplied. If the key cannot be found or if some other part of the call fails a default (English) error message is returned. <BR>
     * Here the get recieves a second argument which is an array of Strings used to populate argument fields, denoted {<I>n</I>}, within the value String returned. Note that argument numbers greater than or equal to 32 are automatically mapped to the formatting String named Farg<I>n</I>.
     * @param key A <strong>String</strong> which is mapped to a initial String within the ResourceBundle.
     * @param args A <strong>String[]</strong> used to populate argument fields within the complete String.
     * @return A <strong>String</strong> which has been referenced by the key String and that either contains no argument fields, or has had the argument fields automatically populated with formatting Strings of with argument String provided in the get call.
     * @see org.greenstone.gatherer.Dictionary
     * @see org.greenstone.gatherer.Gatherer
     */
    private String get(String key, String args[]) {
	if(key.indexOf('.') == -1) {
	    key = "CDM.General.Email." + key;
	}
	return gatherer.dictionary.get(key, args);
    }
    /** The controls used for editing an email address. Basically two text fields with an '@' label in the middle. You also have a title label for this control, which is known for the major email address types, or can be supplied for any other purpose. */
    private class Control 
	extends JPanel {
	/** The at symbol label. */
	private JLabel at = null;
	/** The title label. */
	private JLabel label = null;
	/** The first text field for the user part of an address. */
	private JTextField field1 = null;
	/** The second text field for the host part of an address. */
	private JTextField field2 = null;
	/** The panel on to which the text fields and the at label are placed. */
	private JPanel middle = null;
	/** Constructor.
	 * @param name The <strong>String</strong> to use for the title label.
	 * @see org.greenstone.gatherer.util.EmailAddress.Control.HostListener
	 * @see org.greenstone.gatherer.util.EmailAddress.Control.UserListener
	 */
	public Control(String name) {
	    super();
	    switch(type) {
	    case CREATOR:
		label = new JLabel(get("Creator"));
		break;
	    case MAINTAINER:
		label = new JLabel(get("Maintainer"));
		break;					 
	    default:
		label = new JLabel(name);
	    }
	    middle = new JPanel();
	    field1 = new JTextField(user);
	    field1.setBackground(Color.white);
	    at = new JLabel("@");
	    field2 = new JTextField(host);
	    field2.setBackground(Color.white);
				// Add listeners
	    field1.addKeyListener(new UserListener());
	    field2.addKeyListener(new HostListener());
				// Layout
	    label.setBorder(BorderFactory.createEmptyBorder(2,5,2,10));

	    at.setBorder(BorderFactory.createEmptyBorder(2,5,2,5));

	    middle.setLayout(new BorderLayout());
	    middle.add(field1, BorderLayout.CENTER);
	    middle.add(at, BorderLayout.EAST);

	    setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
	    setLayout(new GridLayout(1,3));
	    add(label);
	    add(middle);
	    add(field2);
	}
	/** Listens for changes of the host part of the email address and updates the email object accordingly. */	 
	private class HostListener
	    extends KeyAdapter {
				/** Whenever a key is released (ie after a key typed event has been fired) when typing in the host field, this method updates the host field of the email object.
				 * @param event A <strong>KeyEvent</strong> containing extra information about the key release performed.
				 */
	    public void keyReleased(KeyEvent event) {
		host = field2.getText();
	    }
	}
	/** Listens for changes of the user part of the email address and updates the email object accordingly. */
	private class UserListener
	    extends KeyAdapter {
				/** Whenever a key is released (ie after a key typed event has been fired) when typing in the user field, this method updates the user field of the email object.
				 * @param event A <strong>KeyEvent</strong> containing extra information about the key release performed.
				 */
	    public void keyReleased(KeyEvent event) {
		user = field1.getText();
	    }
	}
		  
    }
}
