/*
 *    DocumentAction.java
 *    Copyright (C) 2002 New Zealand Digital Library, http://www.nzdl.org
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.greenstone.gsdl3.action;

// Greenstone classes
import org.greenstone.gsdl3.core.ModuleInterface;
import org.greenstone.gsdl3.util.*;

// XML classes
import org.w3c.dom.Document; 
import org.w3c.dom.Element; 
import org.w3c.dom.Node; 
import org.w3c.dom.Text;
import org.w3c.dom.NodeList;

// General Java classes
import java.util.HashMap;
import java.util.HashSet;
import java.io.File;


/** Action class for retrieving Documents  via the message router
 */
public class DocumentAction extends Action {
  
    // this is used to specify that the sibling nodes of a selected one should be obtained
    public static final String SIBLING_ARG = "sib";
    public static final String DOC_TYPE_ARG = "dt";
    public static final String GOTO_PAGE_ARG = "gp";
    public static final String ENRICH_DOC_ARG = "ed";
    
    /** if this is set to true, when a document is displayed, any annotation
     * type services (enrich) will be offered to the user as well */
    protected static final boolean provide_annotations = false; //true;

    public Element process (Element message)
    {
	// for now, no subaction eventually we may want to have subactions such as text assoc or something ?
	
	// the response
	Element result = doc_.createElement(GSXML.MESSAGE_ELEM);
	Element page_response = doc_.createElement(GSXML.RESPONSE_ELEM);
	result.appendChild(page_response);

	// get the request - assume only one
	Element request = (Element)GSXML.getChildByTagName(message, GSXML.REQUEST_ELEM);
	Element cgi_paramList = (Element)GSXML.getChildByTagName(request, GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	HashMap params = GSXML.extractParams(cgi_paramList, false);

	String collection = (String) params.get(GSCGI.COLLECTION_ARG);
	String lang = request.getAttribute(GSXML.LANG_ATT);
	String document_name = (String) params.get(GSCGI.DOCUMENT_ARG);
	if (document_name == null || document_name.equals("")) {
	    System.err.println("DocumentAction Error: no document specified!");
	    return result;
	}
	String document_type = (String) params.get(DOC_TYPE_ARG);
	if (document_type == null) {
	    document_type = "simple";
	}
	//whether to retrieve siblings or not
	boolean get_siblings = false;
	String sibs = (String) params.get(SIBLING_ARG);
	if (sibs != null && sibs.equals("1")) {
	    get_siblings = true;
	}
	
	String sibling_num = (String) params.get(GOTO_PAGE_ARG);
	if (sibling_num != null && !sibling_num.equals("")) {
	    // we have to modify the doc name
	    document_name = document_name+"."+sibling_num+".ss";
	}
	// get the additional data needed for the page
	getBackgroundData(page_response, collection, lang);
	Element format_elem = (Element)GSXML.getChildByTagName(page_response, GSXML.FORMAT_ELEM);
	
	// the_document is where all the doc info - structure and metadata etc
	// is added into, to be returned in the page
	Element the_document = doc_.createElement(GSXML.DOCUMENT_ELEM);
	page_response.appendChild(the_document);

	// set the doctype from the cgi arg as an attribute
	the_document.setAttribute(GSXML.DOC_TYPE_ATT, document_type);

	// create a basic doc list containing the current node
	Element basic_doc_list = doc_.createElement(GSXML.DOC_NODE_ELEM+GSXML.LIST_MODIFIER);
	Element current_doc = doc_.createElement(GSXML.DOC_NODE_ELEM);
	basic_doc_list.appendChild(current_doc);
	current_doc.setAttribute(GSXML.NODE_ID_ATT, document_name);
	
	// Create a parameter list to specify the required structure information
	Element ds_param_list = doc_.createElement(GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);

	Element ds_param = null; 
	boolean get_structure = false;
	boolean get_structure_info = false;
	if (document_type.equals("paged")) {
	    get_structure_info = true;
	    // get teh info needed for paged naviagtion
	    ds_param = doc_.createElement(GSXML.PARAM_ELEM);
	    ds_param_list.appendChild(ds_param);
	    ds_param.setAttribute(GSXML.NAME_ATT, "info");
	    ds_param.setAttribute(GSXML.VALUE_ATT, "numSiblings");
	    ds_param = doc_.createElement(GSXML.PARAM_ELEM);
	    ds_param_list.appendChild(ds_param);
	    ds_param.setAttribute(GSXML.NAME_ATT, "info");
	    ds_param.setAttribute(GSXML.VALUE_ATT, "numChildren");
	    ds_param = doc_.createElement(GSXML.PARAM_ELEM);
	    ds_param_list.appendChild(ds_param);
	    ds_param.setAttribute(GSXML.NAME_ATT, "info");
	    ds_param.setAttribute(GSXML.VALUE_ATT, "siblingPosition");
	    
	} else if (document_type.equals("hierarchy")){
	    get_structure = true;
	    // get the info needed for table of contents
	    ds_param = doc_.createElement(GSXML.PARAM_ELEM);
	    ds_param_list.appendChild(ds_param);
	    ds_param.setAttribute(GSXML.NAME_ATT, "structure");
	    ds_param.setAttribute(GSXML.VALUE_ATT, "ancestors");
	    ds_param = doc_.createElement(GSXML.PARAM_ELEM);
	    ds_param_list.appendChild(ds_param);
	    ds_param.setAttribute(GSXML.NAME_ATT, "structure");
	    ds_param.setAttribute(GSXML.VALUE_ATT, "children");
	    if (get_siblings) {
		ds_param = doc_.createElement(GSXML.PARAM_ELEM);
		ds_param_list.appendChild(ds_param);
		ds_param.setAttribute(GSXML.NAME_ATT, "structure");
		ds_param.setAttribute(GSXML.VALUE_ATT, "siblings");
	    }
	} else {
	    // we dont need any structure
	}

	boolean has_dummy = false;
	if (get_structure || get_structure_info) {

	    // Build a request to obtain the document structure
	    Element ds_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	    String to = GSPath.appendLink(collection, "DocumentStructureRetrieve");// Hard-wired?
	    Element ds_request = GSXML.createBasicRequest(doc_,GSXML.REQUEST_TYPE_PROCESS, to, lang); 
	    ds_message.appendChild(ds_request);
	    ds_request.appendChild(ds_param_list);
	    
	    // create a doc_node_list and put in the doc_node that we are interested in
	    ds_request.appendChild(basic_doc_list);
	    
	    // Process the document structure retrieve message
	    Element ds_response_message = (Element) mr_.process(ds_message);
	    
	    // get the info and print out
	    String path = GSPath.appendLink(GSXML.RESPONSE_ELEM, GSXML.DOC_NODE_ELEM+GSXML.LIST_MODIFIER);
	    path = GSPath.appendLink(path, GSXML.DOC_NODE_ELEM);
	    path = GSPath.appendLink(path, "nodeStructureInfo");
	    Element ds_response_struct_info = (Element) GSXML.getNodeByPath(ds_response_message, path);
	    // get the doc_node bit 
	    if (ds_response_struct_info != null) {
		the_document.appendChild(doc_.importNode(ds_response_struct_info, true)); 
	    }
	    path = GSPath.appendLink(GSXML.RESPONSE_ELEM, GSXML.DOC_NODE_ELEM+GSXML.LIST_MODIFIER);
	    path = GSPath.appendLink(path, GSXML.DOC_NODE_ELEM);
	    path = GSPath.appendLink(path, GSXML.NODE_STRUCTURE_ELEM);
	    Element ds_response_structure = (Element) GSXML.getNodeByPath(ds_response_message, path);
	    
	    if (ds_response_structure != null) {
		// add the contents of the structure bit into the_document
		NodeList structs = ds_response_structure.getChildNodes();
		for (int i=0; i<structs.getLength();i++) {
		    the_document.appendChild(doc_.importNode(structs.item(i), true));
		}
	    } else {
		// no structure nodes, so put in a dummy doc node
		Element doc_node = doc_.createElement(GSXML.DOC_NODE_ELEM);
		doc_node.setAttribute(GSXML.NODE_ID_ATT, document_name);
		the_document.appendChild(doc_node);
		has_dummy = true;
	    }
	} else { // a simple type - we dont have a dummy node for simple
	    // should think about this more
	    // no structure request, so just put in a dummy doc node
	    Element doc_node = doc_.createElement(GSXML.DOC_NODE_ELEM);
	    doc_node.setAttribute(GSXML.NODE_ID_ATT, document_name);
	    the_document.appendChild(doc_node);
	    has_dummy = true;
	}
	
	// Build a request to obtain some document metadata
	Element dm_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	String to = GSPath.appendLink(collection, "DocumentMetadataRetrieve");  // Hard-wired?
	Element dm_request = GSXML.createBasicRequest(doc_, GSXML.REQUEST_TYPE_PROCESS, to, lang);
	dm_message.appendChild(dm_request);
	// Create a parameter list to specify the required metadata information
	
	HashSet meta_names = new HashSet();
	meta_names.add("Title"); // the default
	if (format_elem != null) {
	    extractMetadataNames(format_elem, meta_names);
	}
	
	Element dm_param_list = createMetadataParamList(meta_names);
	dm_request.appendChild(dm_param_list);
	
	// create the doc node list for the metadata request
	Element dm_doc_list = doc_.createElement(GSXML.DOC_NODE_ELEM+GSXML.LIST_MODIFIER);
	dm_request.appendChild(dm_doc_list);

	// Add each node from the structure response into the metadata request
	NodeList doc_nodes = the_document.getElementsByTagName(GSXML.DOC_NODE_ELEM);
	for (int i = 0; i < doc_nodes.getLength(); i++) {
	    Element doc_node = (Element) doc_nodes.item(i);
	    String doc_node_id = doc_node.getAttribute(GSXML.NODE_ID_ATT);

	    // Add the documentNode to the list
	    Element dm_doc_node = doc_.createElement(GSXML.DOC_NODE_ELEM);
	    dm_doc_list.appendChild(dm_doc_node);
	    dm_doc_node.setAttribute(GSXML.NODE_ID_ATT, doc_node_id);
	    dm_doc_node.setAttribute(GSXML.NODE_TYPE_ATT,
				     doc_node.getAttribute(GSXML.NODE_TYPE_ATT));
	}

	// we also want a metadata request to the top level document to get
	// assocfilepath - this could be cached too
	Element doc_meta_request = GSXML.createBasicRequest(doc_, GSXML.REQUEST_TYPE_PROCESS, to, lang);
	dm_message.appendChild(doc_meta_request);
	Element doc_meta_param_list = doc_.createElement(GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	doc_meta_request.appendChild(doc_meta_param_list);
	Element doc_param = doc_.createElement(GSXML.PARAM_ELEM);
	doc_meta_param_list.appendChild(doc_param);
	doc_param.setAttribute(GSXML.NAME_ATT, "metadata");
	doc_param.setAttribute(GSXML.VALUE_ATT, "archivedir");

	// create the doc node list for the metadata request
	Element doc_list = doc_.createElement(GSXML.DOC_NODE_ELEM+GSXML.LIST_MODIFIER);
	doc_meta_request.appendChild(doc_list);

	Element doc_node = doc_.createElement(GSXML.DOC_NODE_ELEM);
	// teh node we want is the root document node
	doc_node.setAttribute(GSXML.NODE_ID_ATT, document_name+".rt");
	doc_list.appendChild(doc_node);
	Element dm_response_message = (Element) mr_.process(dm_message);

	String path = GSPath.appendLink(GSXML.RESPONSE_ELEM, GSXML.DOC_NODE_ELEM+GSXML.LIST_MODIFIER);
	Element dm_response_doc_list = (Element) GSXML.getNodeByPath(dm_response_message, path);

	// Merge the metadata with the structure information
	NodeList dm_response_docs = dm_response_doc_list.getChildNodes();
	for (int i = 0; i < doc_nodes.getLength(); i++) {
	    GSXML.mergeMetadataLists(doc_nodes.item(i), dm_response_docs.item(i));
	}
	// get teh top level doc metadata out
	Element doc_meta_response = (Element)dm_response_message.getElementsByTagName(GSXML.RESPONSE_ELEM).item(1);
	Element doc_meta_list = (Element)GSXML.getNodeByPath(doc_meta_response, "documentNodeList/documentNode/metadataList");
	the_document.appendChild(doc_.importNode(doc_meta_list, true));

	// Build a request to obtain some document content
	Element dc_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	to = GSPath.appendLink(collection, "DocumentContentRetrieve");  // Hard-wired?
	Element dc_request = GSXML.createBasicRequest(doc_, GSXML.REQUEST_TYPE_PROCESS, to, lang);
	dc_message.appendChild(dc_request);
	

	// Create a parameter list to specify the request parameters - empty for now
	Element dc_param_list = doc_.createElement(GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	dc_request.appendChild(dc_param_list);

	// the doc list for the content request is the same as the one for the structure request
	dc_request.appendChild(basic_doc_list);

	Element dc_response_message = (Element) mr_.process(dc_message);
	path = GSPath.appendLink(path, GSXML.DOC_NODE_ELEM);
	Element dc_response_doc = (Element) GSXML.getNodeByPath(dc_response_message, path);
	Element dc_response_doc_content = (Element) GSXML.getChildByTagName(dc_response_doc, GSXML.NODE_CONTENT_ELEM);
	
		    
	boolean highlight_query_terms = true;
	if (highlight_query_terms) {
	    dc_response_doc.removeChild(dc_response_doc_content);
	    
	    dc_response_doc_content = highlightQueryTerms(request, dc_response_doc_content);
	    dc_response_doc.appendChild(dc_response_doc.getOwnerDocument().importNode(dc_response_doc_content, true));
	}
	if (provide_annotations) {
	    String service_selected = (String)params.get(ENRICH_DOC_ARG);
	    if (service_selected != null && service_selected.equals("1")) {
		// now we can modifiy the response doc if needed
		String enrich_service = (String)params.get(GSCGI.SERVICE_ARG);
		System.out.println("enrich service is "+enrich_service);
		// send a message to the service
		Element enrich_message = doc_.createElement(GSXML.MESSAGE_ELEM);
		Element enrich_request = GSXML.createBasicRequest(doc_, GSXML.REQUEST_TYPE_PROCESS, enrich_service, lang);
		enrich_message.appendChild(enrich_request);
		// check for parameters
		HashMap service_params = (HashMap)params.get("s1");
		if (service_params != null) { 
		    Element enrich_pl = GSXML.createParameterList(doc_, service_params); 
		    enrich_request.appendChild(enrich_pl);
		}
		Element e_doc_list = doc_.createElement(GSXML.DOC_NODE_ELEM+GSXML.LIST_MODIFIER);
		enrich_request.appendChild(e_doc_list);
		e_doc_list.appendChild(doc_.importNode(dc_response_doc, true));
		
		Element enrich_response = mr_.process(enrich_message);
		
		String [] links = {GSXML.RESPONSE_ELEM, GSXML.DOC_NODE_ELEM+GSXML.LIST_MODIFIER, GSXML.DOC_NODE_ELEM, GSXML.NODE_CONTENT_ELEM};
		path = GSPath.createPath(links);
		dc_response_doc_content = (Element)GSXML.getNodeByPath(enrich_response, path);
		
	    } 
	}
	// use the returned id rather than the sent one cos there may have
	// been modifiers such as .pr that are removed.
	String modified_doc_id = dc_response_doc.getAttribute(GSXML.NODE_ID_ATT);
	the_document.setAttribute("selectedNode", modified_doc_id);
	if (has_dummy) {
	    // change the id if necessary and add the content
	    Element dummy_node = (Element)doc_nodes.item(0);
	    
	    dummy_node.setAttribute(GSXML.NODE_ID_ATT, modified_doc_id);
	    dummy_node.appendChild(doc_.importNode(dc_response_doc_content, true));
	    // hack for simple type
	    if (document_type.equals("simple")) {
		// we dont want the internal docNode, just want the content and metadata in the document
		// rethink this!!
		the_document.removeChild(dummy_node);

		NodeList dummy_children = dummy_node.getChildNodes();
		//for (int i=0; i<dummy_children.getLength(); i++) {
		for (int i=dummy_children.getLength()-1; i>=0; i--) {
		    the_document.appendChild(dummy_children.item(i));
		    
		}
	    }
	} else {
	    // Merge the document content with the metadata and structure information
	    for (int i = 0; i < doc_nodes.getLength(); i++) {
		Node dn = doc_nodes.item(i);
		String dn_id = ((Element)dn).getAttribute(GSXML.NODE_ID_ATT);
		if (dn_id.equals(modified_doc_id)) {
		    dn.appendChild(doc_.importNode(dc_response_doc_content, true));
		    break;
		}
	    }
	}

	///ystem.out.println("(DocumentAction) Page:\n" + converter_.getPrettyString(result));
	return result;
    }

    /** tell the param class what its arguments are 
     * if an action has its own arguments, this should add them to the params
     * object - particularly important for args that should not be saved */
    public boolean getActionParameters(GSParams params) {
	params.addParameter(DOC_TYPE_ARG, true);
	params.addParameter(GOTO_PAGE_ARG, false);
	params.addParameter(ENRICH_DOC_ARG, false);
 	return true;
    }


    /** this method gets the collection description, the format info, the
     * list of enrich services, etc - stuff that is needed for the page,
     * but is the same whatever the query is - should be cached */
    protected  boolean getBackgroundData(Element page_response, 
					 String collection, String lang) {

	// create a message to process - contains requests for the collection 
	// description, the format element, the enrich services on offer
	// these could all be cached
	Element info_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	String path = GSPath.appendLink(collection, "DocumentMetadataRetrieve");
	// the format request - ignore for now, where does this request go to??
	Element format_request = GSXML.createBasicRequest(doc_, GSXML.REQUEST_TYPE_FORMAT, path, lang);
	info_message.appendChild(format_request);

	// the enrich_services request - only do this if provide_annotations is true

	if (provide_annotations) {
	    Element enrich_services_request = GSXML.createBasicRequest(doc_, GSXML.REQUEST_TYPE_DESCRIBE, "", lang);
	    enrich_services_request.setAttribute(GSXML.INFO_ATT, "serviceList");
	    info_message.appendChild(enrich_services_request);
	}
	
	Element info_response = (Element)mr_.process(info_message);

	// the collection is the first response
	NodeList responses = info_response.getElementsByTagName(GSXML.RESPONSE_ELEM);
	Element format_resp = (Element) responses.item(0);
	
	Element format_elem = (Element)GSXML.getChildByTagName(format_resp, GSXML.FORMAT_ELEM);
	if (format_elem != null) {
	    ///ystem.out.println("doc action found a format statement");
	    // set teh format type
	    format_elem.setAttribute(GSXML.TYPE_ATT, "display");  
	    page_response.appendChild(doc_.importNode(format_elem, true));
	}

	if (provide_annotations) {
	    Element services_resp = (Element)responses.item(1); 

	    // a new message for the mr
	    Element enrich_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	    
	    NodeList e_services = services_resp.getElementsByTagName(GSXML.SERVICE_ELEM);
	    boolean service_found = false;
	    for (int j=0; j<e_services.getLength(); j++) {
		if (((Element)e_services.item(j)).getAttribute(GSXML.TYPE_ATT).equals("enrich")) {
		    Element s = GSXML.createBasicRequest(doc_, GSXML.REQUEST_TYPE_DESCRIBE, ((Element)e_services.item(j)).getAttribute(GSXML.NAME_ATT), lang);
		    enrich_message.appendChild(s);
		    service_found = true;
		}
	    }
	    if (service_found) {
		Element enrich_response = mr_.process(enrich_message);

		NodeList e_responses = enrich_response.getElementsByTagName(GSXML.RESPONSE_ELEM);
		Element service_list = doc_.createElement(GSXML.SERVICE_ELEM + GSXML.LIST_MODIFIER);
		for (int i=0; i<e_responses.getLength(); i++) {
		    Element e_resp = (Element)e_responses.item(i);
		    Element e_service = (Element)doc_.importNode(GSXML.getChildByTagName(e_resp, GSXML.SERVICE_ELEM), true); 
		    e_service.setAttribute(GSXML.NAME_ATT, e_resp.getAttribute(GSXML.FROM_ATT));
		    service_list.appendChild(e_service);
		}
		page_response.appendChild(service_list);
	    }
	} // if provide_annotations
	return true;
	    
    }

    /** this involves a bit of a hack to get the equivalent query terms - has to requery the query service - uses the last selected service name. (if it ends in query). should this action do the query or should it send a message to the query action? but that will involve lots of extra stuff. also doesn't handle phrases properly - just highlights all the terms found in the text.
     */
    protected Element highlightQueryTerms(Element request, Element dc_response_doc_content) {

	// do the query again to get term info 
	Element cgi_param_list = (Element)GSXML.getChildByTagName(request, GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	HashMap params = GSXML.extractParams(cgi_param_list, false);
	
	String service_name = (String)params.get(GSCGI.SERVICE_ARG);
	if (service_name == null || !service_name.endsWith("Query")) { // hack for now - we only do highlighting if we were in a query last - ie not if we were in a browse thingy
	    return dc_response_doc_content;
	}
	String collection = (String)params.get(GSCGI.COLLECTION_ARG);
	String lang = request.getAttribute(GSXML.LANG_ATT);
	String to = GSPath.appendLink(collection, service_name);
	
	Element mr_query_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	Element mr_query_request = GSXML.createBasicRequest(doc_, GSXML.REQUEST_TYPE_PROCESS, to, lang);
	mr_query_message.appendChild(mr_query_request);
	
	// paramList
	Element query_param_list = (Element)doc_.importNode(cgi_param_list, true);
	mr_query_request.appendChild(query_param_list);

	// do the query
       	Element mr_query_response = (Element)mr_.process(mr_query_message);

	String path = GSPath.appendLink(GSXML.RESPONSE_ELEM, GSXML.TERM_ELEM+GSXML.LIST_MODIFIER);
	Element query_term_info_list = (Element) GSXML.getNodeByPath(mr_query_response, path);
	if (query_term_info_list == null) {
	    // no term info
	    System.err.println("DocumentAction: Warning: No query term information.\n");
	    return dc_response_doc_content;
	} 

	NodeList equivs = query_term_info_list.getElementsByTagName("equivTermList");
	HashSet all_terms = new HashSet();
	for (int i=0; i<equivs.getLength(); i++) {
	    
	    // get the terms
	    String [] terms = GSXML.getAttributeValuesFromList((Element)equivs.item(i), GSXML.NAME_ATT);
	    for (int j=0; j<terms.length; j++) {
		
		all_terms.add(terms[j]);
	    }
	}

	Element new_content_elem = doc_.createElement(GSXML.NODE_CONTENT_ELEM);

	String content = GSXML.getNodeText(dc_response_doc_content);

	StringBuffer temp = new StringBuffer();
	StringBuffer temp_content = new StringBuffer();

	for (int i=0; i<content.length(); i++) {
	    char c = content.charAt(i);
	    if (Character.isLetterOrDigit(c)) {
		// not word boundary
		temp.append(c);
	    } else {
		// word boundary
		// add the last word if there was one
		if (temp.length()>0) {
		    if (all_terms.contains(temp.toString())) {
			//if there is anything already present in temp_content, add it as a text node
			Text t = doc_.createTextNode(temp_content.toString());
			new_content_elem.appendChild(t);
			temp_content.delete(0, temp_content.length());
			Element annot = GSXML.createTextElement(doc_, "annotation", temp.toString());
			annot.setAttribute("type", "query_term");
			new_content_elem.appendChild(annot);
			//new_content.append("<annotation type='query_term'>"+temp+"</annotation>");
		    } else {
			temp_content.append(temp);
		    }
		    temp.delete(0, temp.length());
		}
		if (c=='<') {
		    temp_content.append(c);
		    i++;
		    // skip over html
		    while (i<content.length() && content.charAt(i)!='>') {
			temp_content.append(content.charAt(i));
			i++;
		    }
		    temp_content.append(content.charAt(i));
		    //temp_content.append(GSXML.xmlSafe(temp.toString()));
		    //temp.delete(0, temp.length());
		    
		} else {
		    temp_content.append(c);
		}
	    }
	}
	// append anything left of temp_content and temp
	Text t = doc_.createTextNode(temp_content.toString());
	new_content_elem.appendChild(t);

	if (temp.length() > 0) {
	    Element annot = GSXML.createTextElement(doc_, "annotation", temp.toString());
	    annot.setAttribute("type", "query_term");
	    new_content_elem.appendChild(annot);
	}
	//String content_string = "<nodeContent>"+new_content.toString()+"</nodeContent>";
	//Element content_elem = converter_.getDOM(content_string).getDocumentElement();
	return new_content_elem;
    }
}
