
/*
 *    GS2MGPPSearch.java
 *    Copyright (C) 2002 New Zealand Digital Library, http://www.nzdl.org
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.greenstone.gsdl3.service;


// Greenstone classes
import org.greenstone.mgpp.*;
import org.greenstone.gsdl3.util.*;

// XML classes
import org.w3c.dom.Element; 
import org.w3c.dom.Node; 
import org.w3c.dom.NodeList; 

// General Java classes
import java.io.File;
import java.util.HashMap;
import java.util.Vector;
import java.util.Set;
import java.util.Map;
import java.util.Iterator;


/**
 * A ServiceRack class for searching in greenstone 2 MGPP collections
 *
 * @author <a href="mailto:kjdon@cs.waikato.ac.nz">Katherine Don</a>
 * @version $Revision: 4903 $
 * @see ServiceRack
 */
public class GS2MGPPSearch 
    extends GS2Search {

    // extra services offered by mgpp collections
    private static final String FIELD_QUERY_SERVICE = "FieldQuery";
    private static final String ADVANCED_FIELD_QUERY_SERVICE = "AdvancedFieldQuery";

    // extra parameters used
    private static final String INDEX_FIELD_PARAM = "index";
    private static final String LEVEL_PARAM = "level";
    private static final String RANK_PARAM = "sortBy";
    private static final String RANK_PARAM_RANK = "1";
    private static final String RANK_PARAM_NONE = "0";
    private static final String SIMPLE_FIELD_PARAM = "simpleField";
    private static final String ADVANCED_FIELD_PARAM = "complexField";
    // more params for field query
    private static final String FIELD_QUERY_PARAM = "fqv";
    private static final String FIELD_STEM_PARAM = "fqs";
    private static final String FIELD_CASE_PARAM = "fqc";
    private static final String FIELD_FIELD_PARAM = "fqf";
    private static final String FIELD_COMBINE_PARAM = "fqk";
    private static final String FIELD_COMBINE_PARAM_AND = "0";
    private static final String FIELD_COMBINE_PARAM_OR = "1";
    private static final String FIELD_COMBINE_PARAM_NOT = "2";

    private static final int TEXT_QUERY = 0;
    private static final int SIMPLE_QUERY = 1;
    private static final int ADVANCED_QUERY = 2;
    
    protected static final String FIELD_ATT = "field";
    private MGPPWrapper mgpp_src_=null;

    // the default level for retrieval - and we'll use it for searching too
    private String default_level_=null;
    
    /** constructor */
    public GS2MGPPSearch()
    {
	mgpp_src_ = new MGPPWrapper();
    }

    
    /** configure this service */
    public boolean configure(Element info, Element extra_info)
    {
	// Do generic configuration
	if (super.configure(info, extra_info) == false)
	    return false;
	
	// Do specific configuration
	System.out.println("Configuring GS2MGPPSearch...");

	// Get the default level out of <defaultLevel> (buildConfig.xml)
	Element def = (Element) GSXML.getChildByTagName(info, DEFAULT_LEVEL_ELEM);
	if (def != null) {
	    default_level_ = def.getAttribute(GSXML.NAME_ATT);
	}
	if (default_level_ == null || default_level_.equals("")) {
	    System.err.println("Error: default level not specified!");
	    return false;
	}

	// the default level is also the level which gdbm is expecting
	// this must not be overwritten
	mgpp_src_.setReturnLevel(default_level_);
	// return term info
	mgpp_src_.setReturnTerms(true);
	// set the default - this may be overwritten by query params
	mgpp_src_.setQueryLevel(default_level_);

	// set up the extra services which are available for this collection
	// check the config info - if there is no field list, then there is no fielded searching
	
	Element field_list = (Element) GSXML.getChildByTagName(info, GSXML.FIELD_ELEM+GSXML.LIST_MODIFIER);
	if (field_list==null) {
	    // nothing more to do
	    return true;
	}
	// else set up the fielded query services
	
	// set up short_service_info_ - for now just has id and type - name will be added in on teh fly
	Element fq_service = doc_.createElement(GSXML.SERVICE_ELEM);
	fq_service.setAttribute(GSXML.TYPE_ATT, GSXML.SERVICE_TYPE_QUERY);
	fq_service.setAttribute(GSXML.NAME_ATT, FIELD_QUERY_SERVICE);
	short_service_info_.appendChild(fq_service);
	
	Element afq_service = doc_.createElement(GSXML.SERVICE_ELEM);
	afq_service.setAttribute(GSXML.TYPE_ATT, GSXML.SERVICE_TYPE_QUERY);
	afq_service.setAttribute(GSXML.NAME_ATT, ADVANCED_FIELD_QUERY_SERVICE);
	short_service_info_.appendChild(afq_service);
	

	//set up format info - the info is the same as that for text query
	Element format_info = (Element)format_info_map_.get(TEXT_QUERY_SERVICE);
	if (format_info != null) {
	    format_info_map_.put(FIELD_QUERY_SERVICE, format_info);
	    format_info_map_.put(ADVANCED_FIELD_QUERY_SERVICE, format_info);
	}
	return true;
    }

    protected Element getServiceDescription(String service_id, String lang) {
	
	if (!service_id.equals(FIELD_QUERY_SERVICE) && !service_id.equals(ADVANCED_FIELD_QUERY_SERVICE)) {
	    return super.getServiceDescription(service_id, lang);
	}
	
	Element service = doc_.createElement(GSXML.SERVICE_ELEM);
	service.setAttribute(GSXML.TYPE_ATT, GSXML.SERVICE_TYPE_QUERY);
	service.setAttribute(GSXML.NAME_ATT, service_id);
	service.appendChild(GSXML.createDisplayTextElement(doc_, GSXML.DISPLAY_TEXT_NAME, getTextString(service_id+".name", lang)));
	service.appendChild(GSXML.createDisplayTextElement(doc_, GSXML.DISPLAY_TEXT_SUBMIT, getTextString(service_id+".submit", lang)));
	Element param_list = doc_.createElement(GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);	
	service.appendChild(param_list);
	if (service_id.equals(FIELD_QUERY_SERVICE)) {
	    
	    createFieldQueryParamList(param_list, lang);
	} else { 
	    createAdvancedFieldQueryParamList(param_list, lang);
	} 
	return service;
	
    }

     /** creates a new param element and adds it to the param list */
    protected void createParameter(String id, Element param_list, String lang) {
	Element param=null;
	
	if (id.equals(LEVEL_PARAM)) {
	    // the level info - read from config file
	    Element level_list = (Element)GSXML.getChildByTagName(config_info_, LEVEL_ELEM+GSXML.LIST_MODIFIER);
	    NodeList levels = level_list.getElementsByTagName(LEVEL_ELEM);
	    int len = levels.getLength();
	    if (len > 1) { // add level param to list only if more than one index specified
		String [] levs  = new String[len];
		String [] lev_names = new String[len];
		for (int i=0; i<len; i++) {
		    levs[i] = ((Element)levels.item(i)).getAttribute(GSXML.NAME_ATT);
		    lev_names[i] = getTextString("level."+levs[i], lang);
		    
		}
		// the first one is the default
		    param = GSXML.createParameterDescription(doc_, LEVEL_PARAM, getTextString("param."+LEVEL_PARAM, lang), GSXML.PARAM_TYPE_ENUM_SINGLE, levs[0], levs, lev_names);
		    
	    }
	} else if (id.equals(RANK_PARAM)) {
	    String [] vals1 = {RANK_PARAM_RANK, RANK_PARAM_NONE };
	    String [] vals1_texts = { getTextString("param."+RANK_PARAM+"."+RANK_PARAM_RANK, lang), getTextString("param."+RANK_PARAM+"."+RANK_PARAM_NONE, lang)};
		
	    param = GSXML.createParameterDescription(doc_, RANK_PARAM, getTextString("param."+RANK_PARAM, lang), GSXML.PARAM_TYPE_ENUM_SINGLE, RANK_PARAM_RANK, vals1, vals1_texts );
	    
	} else if (id.equals(FIELD_QUERY_PARAM)) {
	    param = GSXML.createParameterDescription(doc_, FIELD_QUERY_PARAM, getTextString("param."+FIELD_QUERY_PARAM, lang), GSXML.PARAM_TYPE_STRING, null, null, null);
	    
	    
	} else if (id.equals(FIELD_CASE_PARAM) || id.equals(FIELD_STEM_PARAM)) {
	    String[] bool_ops = {"0", "1"};
	    String[] bool_texts = {getTextString("param.boolean.off", lang),getTextString("param.boolean.on", lang)}; 
	    param = GSXML.createParameterDescription(doc_, id, getTextString("param."+id, lang), GSXML.PARAM_TYPE_BOOLEAN, BOOLEAN_PARAM_ON, bool_ops, bool_texts);
	    
	} else if (id.equals(FIELD_FIELD_PARAM) || id.equals(INDEX_FIELD_PARAM)) {
	    
	    // the field list -  read from config file
	    Element field_list = (Element)GSXML.getChildByTagName(config_info_, GSXML.FIELD_ELEM+GSXML.LIST_MODIFIER);
	    NodeList fields = field_list.getElementsByTagName(GSXML.FIELD_ELEM);
	    int len = fields.getLength();
	    String [] f_names = new String [len];
	    String [] f_texts = new String [len];
	    for (int i=0; i< len;i++) {
		f_names[i] = ((Element)fields.item(i)).getAttribute(GSXML.SHORTNAME_ATT);
		// should these be changed to a text element based on lang?
		// or is the name of a metadata element eg dc:Title its 
		// name in all langs
		f_texts[i] = ((Element)fields.item(i)).getAttribute(GSXML.NAME_ATT);
		
	    }
	    param = GSXML.createParameterDescription(doc_, id, getTextString("param."+id, lang), GSXML.PARAM_TYPE_ENUM_SINGLE, f_names[0], f_names, f_texts );
	    
	} else if (id.equals(FIELD_COMBINE_PARAM)) {
	    
	    String []vals = {FIELD_COMBINE_PARAM_AND, FIELD_COMBINE_PARAM_OR, FIELD_COMBINE_PARAM_NOT};
	    String []val_texts = {getTextString("param."+FIELD_COMBINE_PARAM+"."+FIELD_COMBINE_PARAM_AND, lang), getTextString("param."+FIELD_COMBINE_PARAM+"."+FIELD_COMBINE_PARAM_OR, lang), getTextString("param."+FIELD_COMBINE_PARAM+"."+FIELD_COMBINE_PARAM_NOT, lang)}; 
		
	    
	    param = GSXML.createParameterDescription(doc_, FIELD_COMBINE_PARAM, "", GSXML.PARAM_TYPE_ENUM_SINGLE, FIELD_COMBINE_PARAM_AND, vals, val_texts);
	    param.setAttribute(GSXML.PARAM_IGNORE_POS_ATT, "0");
		
	    
	}
	
	// add the param to the list
	if (param !=null) {
	    param_list.appendChild(param);
	}
	else {
	    super.createParameter(id, param_list, lang);
	}
    }
   

    /** this creates all teh params and appends them to param_list.
     * if display=true it creates the text strings version
     * otherwise it creates the description version
     */
    protected boolean createTextQueryParamList(Element param_list, 
					       
					       String lang)
    {
	// the order they are specified here is the order they appear on 
	// the query form
	//createParameter(INDEX_PARAM, param_list, display, lang); - there is only ever one index now
	createParameter(INDEX_FIELD_PARAM, param_list,  lang);
	createParameter(LEVEL_PARAM, param_list, lang);
	createParameter(CASE_PARAM, param_list, lang);
	createParameter(STEM_PARAM, param_list,  lang);
	createParameter(MATCH_PARAM, param_list, lang);
	createParameter(RANK_PARAM, param_list,  lang);
	createParameter(MAXDOCS_PARAM, param_list,  lang);
	createParameter(QUERY_PARAM, param_list,  lang);

	return true;
    }



    /** this creates all teh params and appends them to param_list.
     * if display=true it creates the text strings version
     * otherwise it creates the description version
     */
    protected boolean createFieldQueryParamList(Element param_list, 
						String lang)
    {
	// add in the index param
	//createParameter(INDEX_PARAM, param_list, display, lang);
	createParameter(LEVEL_PARAM, param_list, lang);
	createParameter(MATCH_PARAM, param_list,  lang);
	createParameter(CASE_PARAM, param_list,  lang);
	createParameter(STEM_PARAM, param_list,  lang);
	createParameter(RANK_PARAM, param_list,  lang);
	createParameter(MAXDOCS_PARAM, param_list,  lang);

	// create a multi param for the fields etc
	// text box, field
	Element multiparam = null;
	Element param=null;
	multiparam = GSXML.createParameterDescription(doc_, SIMPLE_FIELD_PARAM, "", GSXML.PARAM_TYPE_MULTI, null, null, null);
	multiparam.setAttribute("occurs", "4");
	param_list.appendChild(multiparam);
	
	// the components
	createParameter(FIELD_QUERY_PARAM, multiparam,  lang);
	createParameter(FIELD_FIELD_PARAM, multiparam,  lang);
	
	return true;
    }

    protected boolean createAdvancedFieldQueryParamList(Element param_list, 
							String lang) {

	// first do index and level params
	//createParameter(INDEX_PARAM, param_list, display, lang);
	createParameter(LEVEL_PARAM, param_list,  lang);
	createParameter(RANK_PARAM, param_list, lang);
	createParameter(MAXDOCS_PARAM, param_list,  lang);


	// create a multi param for the fields etc
	// text box, stem, case, field

	Element multiparam = null;
	Element param=null;

	multiparam = GSXML.createParameterDescription(doc_, ADVANCED_FIELD_PARAM, "", GSXML.PARAM_TYPE_MULTI, null, null, null);
	multiparam.setAttribute("occurs", "4");
	param_list.appendChild(multiparam);
	
	createParameter(FIELD_COMBINE_PARAM, multiparam,  lang);
	createParameter(FIELD_QUERY_PARAM, multiparam,  lang);
	createParameter(FIELD_CASE_PARAM, multiparam,  lang);
	createParameter(FIELD_STEM_PARAM, multiparam,  lang);
	createParameter(FIELD_FIELD_PARAM, multiparam,  lang);    
	

	return true;
    }

    // the following three functions are needed so the base class can 
    // call the process+SERVICE_NAME methods
    /** process a text query */
    protected Element processTextQuery(Element request) {
	return processAnyQuery(request, TEXT_QUERY);
    }

    /** process a field query */    
    protected Element processFieldQuery(Element request) {
	return processAnyQuery(request, SIMPLE_QUERY);
    }

    /** process an advanced field query */
    protected Element processAdvancedFieldQuery(Element request) {
	return processAnyQuery(request, ADVANCED_QUERY);
    }
    
    /** process a  query */
    protected Element processAnyQuery(Element request, int query_type)
    {

	String service_name=null;
	String empty_query_test_param=null;
	// set up the type specific bits
	switch (query_type) {
	case TEXT_QUERY:
	    service_name = TEXT_QUERY_SERVICE; 
	    empty_query_test_param = QUERY_PARAM;
	    break;
	case SIMPLE_QUERY:
	    service_name = FIELD_QUERY_SERVICE;
	    empty_query_test_param = FIELD_QUERY_PARAM;
	    break;
	case ADVANCED_QUERY:
	    service_name = ADVANCED_FIELD_QUERY_SERVICE;
	    empty_query_test_param = FIELD_QUERY_PARAM;
	    break;
	default:
	    // should never get here
	    System.err.println("GS2MGPPSearch: wrong query type!!");
	    return null;
	}
	
	// Create a new (empty) result message
	Element result = doc_.createElement(GSXML.RESPONSE_ELEM);
	result.setAttribute(GSXML.FROM_ATT, service_name);
	result.setAttribute(GSXML.TYPE_ATT, GSXML.REQUEST_TYPE_PROCESS);
	
	// Get the parameters of the request
	Element param_list = (Element) GSXML.getChildByTagName(request, GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	if (param_list == null) {
	    System.err.println("Error: TextQuery request had no paramList.");
	    return result;  // Return the empty result
	}

	// Process the request parameters
	HashMap params = GSXML.extractParams(param_list, false);

	// Make sure a query has been specified
	String query = (String) params.get(empty_query_test_param);
	if (query == null || query.equals("")) {
	    return result;  // Return the empty result
	}

	// If an field hasn't been specified, use all fields
	String field = (String) params.get(INDEX_FIELD_PARAM); 
	if (field == null) {
	    field = "ZZ";
	}
	
	// set up mgpp_src
	String indexdir = GSFile.collectionBaseDir(site_home_, cluster_name_) +
	    File.separatorChar + GSFile.collectionIndexPath(cluster_name_, default_index_);
	mgpp_src_.loadIndexData(indexdir);
	setStandardQueryParams(params);
	
	// if field search, create the query string
	switch (query_type) {
	case TEXT_QUERY:
	    query = addFieldInfo(query, field);
	    break;
	case SIMPLE_QUERY:
	    query = parseFieldQueryParams(params);
	    break;
	case ADVANCED_QUERY:
	    query = parseAdvancedFieldQueryParams(params);
	    break;
	}
	// run the query
       	mgpp_src_.runQuery(query);
	MGPPQueryResult mqr= mgpp_src_.getQueryResult();

	// build up the response
	
	// Create a metadata list to store information about the query results
	// should we be using metadataList? or something else?
	Element metadata_list = doc_.createElement(GSXML.METADATA_ELEM+GSXML.LIST_MODIFIER); 
	result.appendChild(metadata_list);
	
	// Add a metadata element specifying the number of matching documents
	long totalDocs = mqr.getTotalDocs();
	Element num_matches_elem = doc_.createElement(GSXML.METADATA_ELEM);
	num_matches_elem.setAttribute(GSXML.NAME_ATT, "numDocsMatched");
	num_matches_elem.setAttribute(GSXML.VALUE_ATT, "" + totalDocs);
	metadata_list.appendChild(num_matches_elem);

	// Create a document list to store the matching documents, and add them
	Vector docs = mqr.getDocs();
	Element document_list = doc_.createElement(GSXML.DOC_NODE_ELEM+GSXML.LIST_MODIFIER);
	result.appendChild(document_list);
	for (int d = 0; d < docs.size(); d++) {
	    long docnum = ((MGPPDocInfo) docs.elementAt(d)).num_;
	    String doc_id = gdbm_src_.docnum2Oid(docnum);
	    Element doc_node = createDocumentNodeElement(doc_id);
	    document_list.appendChild(doc_node);
	}

	// Create a term list to store the term information, and add it
	String query_level = (String)params.get(LEVEL_PARAM); // the current query level
	Element term_list = doc_.createElement(GSXML.TERM_ELEM+GSXML.LIST_MODIFIER);
	result.appendChild(term_list);
	Vector terms = mqr.getTerms();
	for (int t = 0; t < terms.size(); t++) {
	    MGPPTermInfo term_info = (MGPPTermInfo) terms.get(t);
	    
	    Element term_elem = doc_.createElement(GSXML.TERM_ELEM);
	    term_elem.setAttribute(GSXML.NAME_ATT, term_info.term_);
	    term_elem.setAttribute(STEM_ATT, "" + term_info.stem_method_);
	    term_elem.setAttribute(FREQ_ATT, "" + term_info.term_freq_);
	    term_elem.setAttribute(NUM_DOCS_MATCH_ATT, "" + term_info.match_docs_);
	    field = term_info.tag_;
	    if (field.equals(query_level)) {
		// ignore
		field = "";
	    }
	    term_elem.setAttribute(FIELD_ATT, field);
	    
	    Vector equiv_terms = term_info.equiv_terms_;
	    Element equiv_term_list = doc_.createElement(EQUIV_TERM_ELEM+GSXML.LIST_MODIFIER);
	    term_elem.appendChild(equiv_term_list);
	    
	    for (int et = 0; et < equiv_terms.size(); et++) {
		String equiv_term = (String) equiv_terms.get(et);

		Element equiv_term_elem = doc_.createElement(GSXML.TERM_ELEM);
		equiv_term_elem.setAttribute(GSXML.NAME_ATT, equiv_term);
		equiv_term_elem.setAttribute(NUM_DOCS_MATCH_ATT, "");
		equiv_term_elem.setAttribute(FREQ_ATT, "");
		equiv_term_list.appendChild(equiv_term_elem);
	    }

	    term_list.appendChild(term_elem);
	}

	return result;
    }

    // should probably use a list rather than map
    protected boolean setStandardQueryParams(HashMap params) {

	Set entries = params.entrySet();
	Iterator i = entries.iterator();
	while (i.hasNext()) {
	    Map.Entry m = (Map.Entry)i.next();
	    String name = (String)m.getKey();
	    String value = (String)m.getValue();
	   
	    if (name.equals(CASE_PARAM)) {
		boolean val = (value.equals(BOOLEAN_PARAM_ON)?true:false);
		mgpp_src_.setCase(val);		
	    } else if (name.equals(STEM_PARAM)) {
		boolean val = (value.equals(BOOLEAN_PARAM_ON)?true:false);
		mgpp_src_.setStem(val);
	    } else if (name.equals(MAXDOCS_PARAM)&& !value.equals("")) {
		int docs = Integer.parseInt(value);
		mgpp_src_.setMaxDocs(docs);
	    } else if (name.equals(LEVEL_PARAM)) {
		mgpp_src_.setQueryLevel(value);
	    } else if (name.equals(MATCH_PARAM)) {
		int mode;
		if (value.equals(MATCH_PARAM_ALL)) mode=1;
		else mode=0;
		mgpp_src_.setMatchMode(mode);
	    } else if (name.equals(RANK_PARAM)) {
		if (value.equals(RANK_PARAM_RANK)) {
		    mgpp_src_.setSortByRank(true);
		} else if (value.equals(RANK_PARAM_NONE)) {
		    mgpp_src_.setSortByRank(false);
		}
	    } // ignore any others
	}
	return true;
    }

    protected String addFieldInfo(String query, String field) {
	if (field.equals("") || field.equals("ZZ")) {
	    return query;
	}
	return "["+query+"]:"+field;
    }
    /** combines all the field params into a single query
     * - for simple field query */
    protected String parseFieldQueryParams(HashMap params) {

	StringBuffer final_query = new StringBuffer(256);
	String text_line = (String)params.get(FIELD_QUERY_PARAM);
	String[] texts = text_line.split(",", -1);
	String field_line = (String)params.get(FIELD_FIELD_PARAM);
	String[] fields = field_line.split(",", -1);
	String combine="&";
	String match = (String)params.get(MATCH_PARAM);
	if (match.equals(MATCH_PARAM_SOME)) {
	    combine = "|";
	}
	
	for (int i=0; i<texts.length; i++) {

	    String q = texts[i].trim();
	    if (!q.equals("")) {
		addQueryElem(final_query, q, fields[i], combine);
	    }
	}

	return final_query.toString();
    }

    /** combines all the field params into a single query
     * - for advanced field query */   
    protected String parseAdvancedFieldQueryParams(HashMap params) {

	StringBuffer final_query = new StringBuffer(256);
	String text_line = (String)params.get(FIELD_QUERY_PARAM);
	String[] texts = text_line.split(",", -1);
	String field_line = (String)params.get(FIELD_FIELD_PARAM);
	String[] fields = field_line.split(",", -1);
	String case_line = (String)params.get(FIELD_CASE_PARAM);
	String[] cases = case_line.split(",", -1);
	String stem_line = (String)params.get(FIELD_STEM_PARAM);
	String[] stems = stem_line.split(",", -1);
	String  combine_line = (String)params.get(FIELD_COMBINE_PARAM);
	String [] combines = combine_line.split(",", -1);
	String combine = "&";
	for (int i=0; i<texts.length; i++) {
	    if (i==0) {// assume first one is blank
		combine = "";
	    } else {
		String x = combines[i]; 
		if (x.equals(FIELD_COMBINE_PARAM_AND)) {
		    combine = "&";
		} else if (x.equals(FIELD_COMBINE_PARAM_OR)) {
		    combine = "|";
		} else if (x.equals(FIELD_COMBINE_PARAM_NOT)) {
		    combine = "!";
		} 
		
	    }

	    String q = texts[i].trim();
	    if (!q.equals("")) {
		q = addStemAndCase(q, stems[i], cases[i]);
		addQueryElem(final_query, q, fields[i], combine);
	    }
	}

	return final_query.toString();
    }

    protected void addQueryElem(StringBuffer s, String q, String f, String c) {

	String combine="";
	if (s.length()>0) {
	    combine = " "+c+" ";
	}
	if (f.equals("")||f.equals("ZZ")) {
	    s.append(combine+q);
	} else {
	    s.append(combine+"["+q+"]:"+f);
	}
    }
    
    protected String addStemAndCase(String q, String s, String c) {
	String mods = "#";
	if (c.equals("1")) {
	    mods += "i";
	} else {
	    mods += "c";
	}
	if (s.equals("1")) {
	    mods += "s";
	} else {
	    mods+= "u";
	}
	StringBuffer temp = new StringBuffer();
	String [] terms = q.split(" ");
	for (int i=0; i<terms.length; i++) {
	    String t = terms[i].trim();
	    
	    if (!t.equals("") && !t.equals("TX")) {
		temp.append(" "+t+mods);
	    }
	}
	return temp.toString();
    }


}
 

